@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <sys/types.h>
@@ -84,10 +84,11 @@ void KateBuffer::setMaxLoadedBlocks (uint count)
 class KateFileLoader
 {
   public:
-    KateFileLoader (const QString &filename, QTextCodec *codec)
+    KateFileLoader (const QString &filename, QTextCodec *codec, bool removeTrailingSpaces)
       : m_file (filename)
       , m_buffer (KMIN (m_file.size(), KATE_FILE_LOADER_BS))
-      , m_decoder (codec->makeDecoder())
+      , m_codec (codec)
+      , m_decoder (m_codec->makeDecoder())
       , m_position (0)
       , m_lastLineStart (0)
       , m_eof (false) // default to not eof
@@ -96,7 +97,9 @@ class KateFileLoader
       , m_eol (-1) // no eol type detected atm
       , m_twoByteEncoding (QString(codec->name()) == "ISO-10646-UCS-2")
       , m_binary (false)
+      , m_removeTrailingSpaces (removeTrailingSpaces)
     {
+      kdDebug (13020) << "OPEN USES ENCODING: " << m_codec->name() << endl;
     }
 
     ~KateFileLoader ()
@@ -115,6 +118,14 @@ class KateFileLoader
 
         if (c > 0)
         {
+          // fix utf16 LE, stolen from khtml ;)
+          if ((c >= 2) && (m_codec->mibEnum() == 1000) && (m_buffer[1] == 0x00))
+          {
+            // utf16LE, we need to put the decoder in LE mode
+            char reverseUtf16[3] = {0xFF, 0xFE, 0x00};
+            m_decoder->toUnicode(reverseUtf16, 2);
+          }
+
           processNull (c);
           m_text = m_decoder->toUnicode (m_buffer, c);
         }
@@ -158,6 +169,9 @@ class KateFileLoader
     // binary ?
     inline bool binary () const { return m_binary; }
 
+    // should spaces be ignored at end of line?
+    inline bool removeTrailingSpaces () const { return m_removeTrailingSpaces; }
+
     // internal unicode data array
     inline const QChar *unicode () const { return m_text.unicode(); }
 
@@ -289,6 +303,7 @@ class KateFileLoader
   private:
     QFile m_file;
     QByteArray m_buffer;
+    QTextCodec *m_codec;
     QTextDecoder *m_decoder;
     QString m_text;
     uint m_position;
@@ -299,6 +314,7 @@ class KateFileLoader
     int m_eol;
     bool m_twoByteEncoding;
     bool m_binary;
+    bool m_removeTrailingSpaces;
 };
 
 /**
@@ -310,6 +326,8 @@ KateBuffer::KateBuffer(KateDocument *doc)
    editIsRunning (false),
    editTagLineStart (0xffffffff),
    editTagLineEnd (0),
+   editTagLineFrom (false),
+   editChangesDone (false),
    m_doc (doc),
    m_lines (0),
    m_lastInSyncBlock (0),
@@ -353,6 +371,9 @@ void KateBuffer::editStart ()
 
   editTagLineStart = 0xffffffff;
   editTagLineEnd = 0;
+  editTagLineFrom = false;
+
+  editChangesDone = false;
 }
 
 void KateBuffer::editEnd ()
@@ -365,78 +386,48 @@ void KateBuffer::editEnd ()
   if (editSessionNumber > 0)
     return;
 
-  // hl update !!!
-  if ( m_highlight && !m_highlight->noHighlighting()
-       && (editTagLineStart <= editTagLineEnd)
-       && (editTagLineEnd <= m_lineHighlighted))
+  if (editChangesDone)
   {
-    // look one line too far, needed for linecontinue stuff
-    editTagLineEnd++;
-
-    // look one line before, needed nearly 100% only for indentation based folding !
-    if (editTagLineStart > 0)
-      editTagLineStart--;
-
-    KateBufBlock *buf2 = 0;
-    bool needContinue = false;
-    while ((buf2 = findBlock(editTagLineStart)))
+    // hl update !!!
+    if ( m_highlight && !m_highlight->noHighlighting()
+        && (editTagLineStart <= editTagLineEnd)
+        && (editTagLineEnd <= m_lineHighlighted))
     {
-      needContinue = doHighlight (buf2,
-        (editTagLineStart > buf2->startLine()) ? editTagLineStart : buf2->startLine(),
-        (editTagLineEnd > buf2->endLine()) ? buf2->endLine() : editTagLineEnd,
-        true);
+      // look one line too far, needed for linecontinue stuff
+      editTagLineEnd++;
 
-      editTagLineStart = (editTagLineEnd > buf2->endLine()) ? buf2->endLine() : editTagLineEnd;
+      // look one line before, needed nearly 100% only for indentation based folding !
+      if (editTagLineStart > 0)
+        editTagLineStart--;
 
-      if ((editTagLineStart >= m_lines) || (editTagLineStart >= editTagLineEnd))
-        break;
+      KateBufBlock *buf2 = 0;
+      bool needContinue = false;
+      while ((buf2 = findBlock(editTagLineStart)))
+      {
+        needContinue = doHighlight (buf2,
+          (editTagLineStart > buf2->startLine()) ? editTagLineStart : buf2->startLine(),
+          (editTagLineEnd > buf2->endLine()) ? buf2->endLine() : editTagLineEnd,
+          true);
+
+        editTagLineStart = (editTagLineEnd > buf2->endLine()) ? buf2->endLine() : editTagLineEnd;
+
+        if ((editTagLineStart >= m_lines) || (editTagLineStart >= editTagLineEnd))
+          break;
+      }
+
+      if (needContinue)
+        m_lineHighlighted = editTagLineStart;
+
+      if (editTagLineStart > m_lineHighlightedMax)
+        m_lineHighlightedMax = editTagLineStart;
     }
-
-    if (needContinue)
-      m_lineHighlighted = editTagLineStart;
-
-    if (editTagLineStart > m_lineHighlightedMax)
+    else if (editTagLineStart < m_lineHighlightedMax)
       m_lineHighlightedMax = editTagLineStart;
   }
-  else if (editTagLineStart < m_lineHighlightedMax)
-    m_lineHighlightedMax = editTagLineStart;
 
   editIsRunning = false;
 }
 
-void KateBuffer::editTagLine (uint line)
-{
-  if (line < editTagLineStart)
-    editTagLineStart = line;
-
-  if (line > editTagLineEnd)
-    editTagLineEnd = line;
-}
-
-void KateBuffer::editInsertTagLine (uint line)
-{
-  if (line < editTagLineStart)
-    editTagLineStart = line;
-
-  if (line <= editTagLineEnd)
-    editTagLineEnd++;
-
-  if (line > editTagLineEnd)
-    editTagLineEnd = line;
-}
-
-void KateBuffer::editRemoveTagLine (uint line)
-{
-  if (line < editTagLineStart)
-    editTagLineStart = line;
-
-  if (line < editTagLineEnd)
-    editTagLineEnd--;
-
-  if (line > editTagLineEnd)
-    editTagLineEnd = line;
-}
-
 void KateBuffer::clear()
 {
   m_regionTree.clear();
@@ -466,7 +457,7 @@ void KateBuffer::clear()
 
 bool KateBuffer::openFile (const QString &m_file)
 {
-  KateFileLoader file (m_file, m_doc->config()->codec());
+  KateFileLoader file (m_file, m_doc->config()->codec(), m_doc->configFlags() & KateDocument::cfRemoveSpaces);
 
   bool ok = false;
   struct stat sbuf;
@@ -482,8 +473,8 @@ bool KateBuffer::openFile (const QString &m_file)
     return false; // Error
   }
 
-  // set eol mode, if a eol char was found in the first 256kb block!
-  if (file.eol() != -1)
+  // set eol mode, if a eol char was found in the first 256kb block and we allow this at all!
+  if (m_doc->config()->allowEolDetection() && (file.eol() != -1))
     m_doc->config()->setEol (file.eol());
 
   // flush current content
@@ -588,61 +579,33 @@ bool KateBuffer::saveFile (const QString &m_file)
   // this line sets the mapper to the correct codec
   stream.setCodec(codec);
 
+  // our loved eol string ;)
   QString eol = m_doc->config()->eolString ();
 
-  // for tab replacement, initialize only once
-  uint pos, found, ml, l;
-  QChar onespace(' ');
-  QString onetab("\t");
-  uint tw = m_doc->config()->tabWidth();
-
-  // Use the document methods
-  if ( m_doc->configFlags() & KateDocument::cfReplaceTabs ||
-       m_doc->configFlags() & KateDocument::cfRemoveSpaces )
-    m_doc->editStart();
+  // should we strip spaces?
+  bool removeTrailingSpaces = m_doc->configFlags() & KateDocument::cfRemoveSpaces;
 
+  // just dump the lines out ;)
   for (uint i=0; i < m_lines; i++)
   {
-    KateTextLine::Ptr textLine = plainLine(i);
+    KateTextLine::Ptr textline = plainLine(i);
 
-    if (textLine)
+    // strip spaces
+    if (removeTrailingSpaces)
     {
-      // replace tabs if required
-      if ( m_doc->configFlags() & KateDocument::cfReplaceTabs )
+      int lastChar = textline->lastChar();
+
+      if (lastChar > -1)
       {
-        pos = 0;
-        while ( textLine->searchText( pos, onetab, &found, &ml ) )
-        {
-          l = tw - ( found%tw );
-          if ( l )
-          {
-            QString t;
-            m_doc->editRemoveText( i, found, 1 );
-            m_doc->editInsertText( i, found, t.fill(onespace, l) ); // ### anything more efficient?
-            pos += l-1;
-          }
-        }
+        stream << QConstString (textline->text(), lastChar+1).string();
       }
-
-      // remove trailing spaces if required
-      if ( (m_doc->configFlags() & KateDocument::cfRemoveSpaces) && textLine->length() )
-      {
-        pos = textLine->length() - 1;
-        uint lns = textLine->lastChar();
-        if ( lns != pos )
-          m_doc->editRemoveText( i, lns + 1, pos - lns );
-      }
-
-      stream << textLine->string();
-
-      if ((i+1) < m_lines)
-        stream << eol;
     }
-  }
+    else // simple, dump the line
+      stream << textline->string();
 
-  if ( m_doc->configFlags() & KateDocument::cfReplaceTabs ||
-       m_doc->configFlags() & KateDocument::cfRemoveSpaces )
-    m_doc->editEnd();
+    if ((i+1) < m_lines)
+      stream << eol;
+  }
 
   file.close ();
 
@@ -740,10 +703,21 @@ void KateBuffer::changeLine(uint i)
 {
   KateBufBlock *buf = findBlock(i);
 
-  editTagLine (i);
+  if (!buf)
+    return;
 
-  if (buf)
-    buf->markDirty ();
+  // mark this block dirty
+  buf->markDirty ();
+
+  // mark buffer changed
+  editChangesDone = true;
+
+  // tag this line as changed
+  if (i < editTagLineStart)
+    editTagLineStart = i;
+
+  if (i > editTagLineEnd)
+    editTagLineEnd = i;
 }
 
 void KateBuffer::insertLine(uint i, KateTextLine::Ptr line)
@@ -776,7 +750,21 @@ void KateBuffer::insertLine(uint i, KateTextLine::Ptr line)
   if (m_lastInSyncBlock < m_lastFoundBlock)
     m_lastFoundBlock = m_lastInSyncBlock;
 
-  editInsertTagLine (i);
+  // mark buffer changed
+  editChangesDone = true;
+
+  // tag this line as inserted
+  if (i < editTagLineStart)
+    editTagLineStart = i;
+
+  if (i <= editTagLineEnd)
+    editTagLineEnd++;
+
+  if (i > editTagLineEnd)
+    editTagLineEnd = i;
+
+  // line inserted
+  editTagLineFrom = true;
 
   m_regionTree.lineHasBeenInserted (i);
 }
@@ -831,7 +819,21 @@ void KateBuffer::removeLine(uint i)
   if (m_lastInSyncBlock < m_lastFoundBlock)
     m_lastFoundBlock = m_lastInSyncBlock;
 
-  editRemoveTagLine (i);
+  // mark buffer changed
+  editChangesDone = true;
+
+  // tag this line as removed
+   if (i < editTagLineStart)
+    editTagLineStart = i;
+
+  if (i < editTagLineEnd)
+    editTagLineEnd--;
+
+  if (i > editTagLineEnd)
+    editTagLineEnd = i;
+
+  // line removed
+  editTagLineFrom = true;
 
   m_regionTree.lineHasBeenRemoved (i);
 }
@@ -885,21 +887,81 @@ void KateBuffer::invalidateHighlighting()
   m_lineHighlighted = 0;
 }
 
+
+void KateBuffer::updatePreviousNotEmptyLine(KateBufBlock *blk,uint current_line,bool addindent,uint deindent)
+{
+  KateTextLine::Ptr textLine;
+  do {
+    if (current_line>0) current_line--;
+    else
+    {
+      uint line=blk->startLine()+current_line;
+      if (line==0) return;
+      line--;
+      blk=findBlock(line);
+      if (!blk) {
+        kdDebug(13020)<<"updatePreviousNotEmptyLine: block not found, this must not happen"<<endl;
+        return;
+      }
+      current_line=line-blk->startLine();
+    }
+    textLine = blk->line(current_line);
+  } while (textLine->firstChar()==-1);
+  kdDebug(13020)<<"updatePreviousNotEmptyLine: updating line:"<<(blk->startLine()+current_line)<<endl;
+  QMemArray<uint> foldingList=textLine->foldingListArray();
+  while ( (foldingList.size()>0)  && ( abs(foldingList[foldingList.size()-2])==1)) {
+    foldingList.resize(foldingList.size()-2,QGArray::SpeedOptim);
+  }
+  addIndentBasedFoldingInformation(foldingList,addindent,deindent);
+  textLine->setFoldingList(foldingList);
+  bool retVal_folding = false;
+  m_regionTree.updateLine (current_line + blk->startLine(), &foldingList, &retVal_folding, true,false);
+  emit tagLines (blk->startLine()+current_line, blk->startLine()+current_line);
+}
+
+void KateBuffer::addIndentBasedFoldingInformation(QMemArray<uint> &foldingList,bool addindent,uint deindent)
+{
+  if (addindent) {
+    //kdDebug(13020)<<"adding indent for line :"<<current_line + buf->startLine()<<"  textLine->noIndentBasedFoldingAtStart"<<textLine->noIndentBasedFoldingAtStart()<<endl;
+    kdDebug(13020)<<"adding ident"<<endl;
+    foldingList.resize (foldingList.size() + 2, QGArray::SpeedOptim);
+    foldingList[foldingList.size()-2] = 1;
+    foldingList[foldingList.size()-1] = 0;
+  }
+  kdDebug(13020)<<"DEINDENT: "<<deindent<<endl;
+  if (deindent > 0)
+  {
+    foldingList.resize (foldingList.size() + (deindent*2), QGArray::SpeedOptim);
+
+    for (uint z= foldingList.size()-(deindent*2); z < foldingList.size(); z=z+2)
+    {
+      foldingList[z] = -1;
+      foldingList[z+1] = 0;
+    }
+  }
+}
+
 bool KateBuffer::doHighlight (KateBufBlock *buf, uint startLine, uint endLine, bool invalidate)
 {
   // no hl around, no stuff to do
   if (!m_highlight)
     return false;
 
+  /*if (m_highlight->foldingIndentationSensitive())
+  {
+    startLine=0;
+    endLine=50;
+  }*/
+
   // we tried to start in a line behind this buf block !
   if (startLine >= (buf->startLine()+buf->lines()))
     return false;
 
-  QTime t;
-  t.start();
-  kdDebug (13020) << "HIGHLIGHTED START --- NEED HL, LINESTART: " << startLine << " LINEEND: " << endLine << endl;
-  kdDebug (13020) << "HL UNTIL LINE: " << m_lineHighlighted << " MAX: " << m_lineHighlightedMax << endl;
-  kdDebug (13020) << "HL DYN COUNT: " << KateHlManager::self()->countDynamicCtxs() << " MAX: " << m_maxDynamicContexts << endl;
+  //QTime t;
+  //t.start();
+  //kdDebug (13020) << "HIGHLIGHTED START --- NEED HL, LINESTART: " << startLine << " LINEEND: " << endLine << endl;
+  //kdDebug (13020) << "HL UNTIL LINE: " << m_lineHighlighted << " MAX: " << m_lineHighlightedMax << endl;
+  //kdDebug (13020) << "HL DYN COUNT: " << KateHlManager::self()->countDynamicCtxs() << " MAX: " << m_maxDynamicContexts << endl;
 
   // see if there are too many dynamic contexts; if yes, invalidate HL of all documents
   if (KateHlManager::self()->countDynamicCtxs() >= m_maxDynamicContexts)
@@ -959,9 +1021,10 @@ bool KateBuffer::doHighlight (KateBufBlock *buf, uint startLine, uint endLine, b
   // here we are atm, start at start line in the block
   uint current_line = startLine - buf->startLine();
 
-  // does we need to continue
+  // do we need to continue
   bool stillcontinue=false;
-
+  bool indentContinueWhitespace=false;
+  bool indentContinueNextWhitespace=false;
   // loop over the lines of the block, from startline to endline or end of block
   // if stillcontinue forces us to do so
   while ( (current_line < buf->lines())
@@ -987,25 +1050,44 @@ bool KateBuffer::doHighlight (KateBufBlock *buf, uint startLine, uint endLine, b
 
       // current indentation of this line
       uint iDepth = textLine->indentDepth(m_tabWidth);
+      if ((current_line+buf->startLine())==0)
+      {
+          indentDepth.resize (1, QGArray::SpeedOptim);
+          indentDepth[0] = iDepth;
+      }
 
-      // this line is empty, beside spaces, use indentation depth of the previous line !
-      if (textLine->firstChar() == -1)
+      textLine->setNoIndentBasedFoldingAtStart(prevLine->noIndentBasedFolding());
+      // this line is empty, beside spaces, or has indentaion based folding disabled, use indentation depth of the previous line !
+      kdDebug(13020)<<"current_line:"<<current_line + buf->startLine()<<" textLine->noIndentBasedFoldingAtStart"<<textLine->noIndentBasedFoldingAtStart()<<endl;
+      if ( (textLine->firstChar() == -1) || textLine->noIndentBasedFoldingAtStart())
       {
         // do this to get skipped empty lines indent right, which was given in the indenation array
         if (!prevLine->indentationDepthArray().isEmpty())
+        {
           iDepth = (prevLine->indentationDepthArray())[prevLine->indentationDepthArray().size()-1];
+          kdDebug(13020)<<"reusing old depth as current"<<endl;
+        }
         else
+        {
           iDepth = prevLine->indentDepth(m_tabWidth);
+          kdDebug(13020)<<"creating indentdepth for previous line"<<endl;
+        }
       }
 
+      kdDebug(13020)<<"iDepth:"<<iDepth<<endl;
+
       // query the next line indentation, if we are at the end of the block
       // use the first line of the next buf block
       uint nextLineIndentation = 0;
-
+      bool nextLineIndentationValid=true;
+      indentContinueNextWhitespace=false;
       if ((current_line+1) < buf->lines())
       {
         if (buf->line(current_line+1)->firstChar() == -1)
+        {
           nextLineIndentation = iDepth;
+          indentContinueNextWhitespace=true;
+        }
         else
           nextLineIndentation = buf->line(current_line+1)->indentDepth(m_tabWidth);
       }
@@ -1016,76 +1098,90 @@ bool KateBuffer::doHighlight (KateBufBlock *buf, uint startLine, uint endLine, b
         if (blk && (blk->lines() > 0))
         {
           if (blk->line (0)->firstChar() == -1)
+          {
             nextLineIndentation = iDepth;
+            indentContinueNextWhitespace=true;
+          }
           else
             nextLineIndentation = blk->line (0)->indentDepth(m_tabWidth);
         }
+        else nextLineIndentationValid=false;
       }
 
-      // recalculate the indentation array for this line, query if we have to add
-      // a new folding start, this means newIn == true !
-      bool newIn = false;
-      if ((iDepth > 0) && (indentDepth.isEmpty() || (indentDepth[indentDepth.size()-1] < iDepth)))
-      {
-        indentDepth.resize (indentDepth.size()+1, QGArray::SpeedOptim);
-        indentDepth[indentDepth.size()-1] = iDepth;
-        newIn = true;
-      }
-      else
-      {
-        for (int z=indentDepth.size()-1; z > -1; z--)
+      if  (!textLine->noIndentBasedFoldingAtStart()) {
+
+        if ((iDepth > 0) && (indentDepth.isEmpty() || (indentDepth[indentDepth.size()-1] < iDepth)))
         {
-          if (indentDepth[z] > iDepth)
-            indentDepth.resize (z, QGArray::SpeedOptim);
-          else if (indentDepth[z] == iDepth)
-            break;
-          else if (indentDepth[z] < iDepth)
+          kdDebug(13020)<<"adding depth to \"stack\":"<<iDepth<<endl;
+          indentDepth.resize (indentDepth.size()+1, QGArray::SpeedOptim);
+          indentDepth[indentDepth.size()-1] = iDepth;
+        } else {
+          if (!indentDepth.isEmpty())
           {
-            indentDepth.resize (indentDepth.size()+1, QGArray::SpeedOptim);
-            indentDepth[indentDepth.size()-1] = iDepth;
-            newIn = true;
-            break;
+            for (int z=indentDepth.size()-1; z > -1; z--)
+              if (indentDepth[z]>iDepth)
+                indentDepth.resize(z, QGArray::SpeedOptim);
+            if ((iDepth > 0) && (indentDepth.isEmpty() || (indentDepth[indentDepth.size()-1] < iDepth)))
+            {
+              kdDebug(13020)<<"adding depth to \"stack\":"<<iDepth<<endl;
+              indentDepth.resize (indentDepth.size()+1, QGArray::SpeedOptim);
+              indentDepth[indentDepth.size()-1] = iDepth;
+              if (prevLine->firstChar()==-1) {
+
+              }
+            }
           }
         }
       }
 
-      // just for debugging always true to start with !
+      if (!textLine->noIndentBasedFolding())
+      {
+        if (nextLineIndentationValid)
+        {
+          //if (textLine->firstChar()!=-1)
+          {
+            kdDebug(13020)<<"nextLineIndentation:"<<nextLineIndentation<<endl;
+            bool addindent=false;
+            uint deindent=0;
+            if (!indentDepth.isEmpty())
+              kdDebug()<<"indentDepth[indentDepth.size()-1]:"<<indentDepth[indentDepth.size()-1]<<endl;
+            if ((nextLineIndentation>0) && ( indentDepth.isEmpty() || (indentDepth[indentDepth.size()-1]<nextLineIndentation)))
+            {
+              kdDebug(13020)<<"addindent==true"<<endl;
+              addindent=true;
+            } else {
+            if ((!indentDepth.isEmpty()) && (indentDepth[indentDepth.size()-1]>nextLineIndentation))
+              {
+                kdDebug(13020)<<"...."<<endl;
+                for (int z=indentDepth.size()-1; z > -1; z--)
+                {
+                  kdDebug(13020)<<indentDepth[z]<<"  "<<nextLineIndentation<<endl;
+                  if (indentDepth[z]>nextLineIndentation)
+                    deindent++;
+                }
+              }
+            }
+/*        }
+        if (textLine->noIndentBasedFolding()) kdDebug(13020)<<"=============================indentation based folding disabled======================"<<endl;
+        if (!textLine->noIndentBasedFolding()) {*/
+            if ((textLine->firstChar()==-1)) {
+              updatePreviousNotEmptyLine(buf,current_line,addindent,deindent);
+              codeFoldingUpdate=true;
+            }
+            else
+            {
+              addIndentBasedFoldingInformation(foldingList,addindent,deindent);
+            }
+          }
+        }
+      }
       indentChanged = !(indentDepth == textLine->indentationDepthArray());
 
       // assign the new array to the textline !
       if (indentChanged)
         textLine->setIndentationDepth (indentDepth);
 
-      // add folding start to the list !
-      if (newIn)
-      {
-        foldingList.resize (foldingList.size() + 2, QGArray::SpeedOptim);
-        foldingList[foldingList.size()-2] = 1;
-        foldingList[foldingList.size()-1] = 0;
-      }
-
-      // calculate how much end folding symbols must be added to the list !
-      // remIn gives you the count of them
-      uint remIn = 0;
-
-      for (int z=indentDepth.size()-1; z > -1; z--)
-      {
-        if (indentDepth[z] > nextLineIndentation)
-          remIn++;
-        else
-          break;
-      }
-
-      if (remIn > 0)
-      {
-        foldingList.resize (foldingList.size() + (remIn*2), QGArray::SpeedOptim);
-
-        for (uint z= foldingList.size()-(remIn*2); z < foldingList.size(); z=z+2)
-        {
-          foldingList[z] = -1;
-          foldingList[z+1] = 0;
-        }
-      }
+      indentContinueWhitespace=textLine->firstChar()==-1;
     }
     bool foldingColChanged=false;
     bool foldingChanged = false; //!(foldingList == textLine->foldingListArray());
@@ -1124,7 +1220,7 @@ bool KateBuffer::doHighlight (KateBufBlock *buf, uint startLine, uint endLine, b
     codeFoldingUpdate = codeFoldingUpdate | retVal_folding;
 
     // need we to continue ?
-    stillcontinue = ctxChanged || indentChanged;
+    stillcontinue = ctxChanged || indentChanged || indentContinueWhitespace || indentContinueNextWhitespace;
 
     // move around the lines
     prevLine = textLine;
@@ -1143,10 +1239,10 @@ bool KateBuffer::doHighlight (KateBufBlock *buf, uint startLine, uint endLine, b
   if (codeFoldingUpdate)
     emit codeFoldingUpdated();
 
-  kdDebug (13020) << "HIGHLIGHTED END --- NEED HL, LINESTART: " << startLine << " LINEEND: " << endLine << endl;
-  kdDebug (13020) << "HL UNTIL LINE: " << m_lineHighlighted << " MAX: " << m_lineHighlightedMax << endl;
-  kdDebug (13020) << "HL DYN COUNT: " << KateHlManager::self()->countDynamicCtxs() << " MAX: " << m_maxDynamicContexts << endl;
-  kdDebug (13020) << "TIME TAKEN: " << t.elapsed() << endl;
+  //kdDebug (13020) << "HIGHLIGHTED END --- NEED HL, LINESTART: " << startLine << " LINEEND: " << endLine << endl;
+  //kdDebug (13020) << "HL UNTIL LINE: " << m_lineHighlighted << " MAX: " << m_lineHighlightedMax << endl;
+  //kdDebug (13020) << "HL DYN COUNT: " << KateHlManager::self()->countDynamicCtxs() << " MAX: " << m_maxDynamicContexts << endl;
+  //kdDebug (13020) << "TIME TAKEN: " << t.elapsed() << endl;
 
   // if we are at the last line of the block + we still need to continue
   // return the need of that !
@@ -1251,6 +1347,18 @@ void KateBufBlock::fillBlock (KateFileLoader *stream)
     stream->readLine(offset, length);
     const QChar *unicodeData = stream->unicode () + offset;
 
+    // strip spaces at end of line
+    if ( stream->removeTrailingSpaces() )
+    {
+      while (length > 0)
+      {
+        if (unicodeData[length-1].isSpace())
+          --length;
+        else
+          break;
+      }
+    }
+
     blockSize += length;
 
     if (swap)
