@@ -728,8 +728,8 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
         "Each new bookmark will be added to the bottom, independently from "
         "where it is placed in the document."));
   QWhatsThis::add(m_showIndentLines, i18n(
-        "If this is enabled, the editor will display vertical line to help "
-        "identifying indent lines.") );
+        "If this is enabled, the editor will display vertical lines to help "
+        "identify indent lines.") );
 
   reload();
 
