@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 //BEGIN Includes
@@ -85,7 +85,7 @@ class KateStyleListItem : public QListViewItem
   public:
     KateStyleListItem( QListViewItem *parent=0, const QString & stylename=0,
                    class KateAttribute* defaultstyle=0, class KateHlItemData *data=0 );
-    KateStyleListItem( QListView *parent=0, const QString & stylename=0,
+    KateStyleListItem( QListView *parent, const QString & stylename=0,
                    class KateAttribute* defaultstyle=0, class KateHlItemData *data=0 );
     ~KateStyleListItem() { if (st) delete is; };
 
@@ -261,17 +261,16 @@ QString KateSchemaManager::name (uint number)
 }
 //END
 
-//
-//
 //
 // DIALOGS !!!
 //
-//
 
-//BEGIN KateSchemaConfigColorTab
+//BEGIN KateSchemaConfigColorTab -- 'Colors' tab
 KateSchemaConfigColorTab::KateSchemaConfigColorTab( QWidget *parent, const char * )
   : QWidget (parent)
 {
+  m_schema = -1;
+
   QHBox *b;
   QLabel *label;
 
@@ -385,8 +384,26 @@ KateSchemaConfigColorTab::~KateSchemaConfigColorTab()
 {
 }
 
-void KateSchemaConfigColorTab::readConfig (KConfig *config)
+void KateSchemaConfigColorTab::schemaChanged ( int newSchema )
 {
+  // save curent schema
+  if ( m_schema > -1 )
+  {
+    m_schemas[ m_schema ].back = m_back->color();
+    m_schemas[ m_schema ].selected = m_selected->color();
+    m_schemas[ m_schema ].current = m_current->color();
+    m_schemas[ m_schema ].bracket = m_bracket->color();
+    m_schemas[ m_schema ].wwmarker = m_wwmarker->color();
+    m_schemas[ m_schema ].iconborder = m_iconborder->color();
+    m_schemas[ m_schema ].tmarker = m_tmarker->color();
+    m_schemas[ m_schema ].linenumber = m_linenumber->color();
+  }
+
+  if ( newSchema == m_schema ) return;
+
+  // switch
+  m_schema = newSchema;
+
   // first disconnect all signals otherwise setColor emits changed
   m_back      ->disconnect( SIGNAL( changed( const QColor & ) ) );
   m_selected  ->disconnect( SIGNAL( changed( const QColor & ) ) );
@@ -398,45 +415,65 @@ void KateSchemaConfigColorTab::readConfig (KConfig *config)
   m_markers   ->disconnect( SIGNAL( changed( const QColor & ) ) );
   m_linenumber->disconnect( SIGNAL( changed( const QColor & ) ) );
 
-  QColor tmp0 (KGlobalSettings::baseColor());
-  QColor tmp1 (KGlobalSettings::highlightColor());
-  QColor tmp2 (KGlobalSettings::alternateBackgroundColor());
-  QColor tmp3 ( "#FFFF99" );
-  QColor tmp4 (tmp2.dark());
-  QColor tmp5 ( KGlobalSettings::textColor() );
-  QColor tmp6 ( "#EAE9E8" );
-  QColor tmp7 ( "#000000" );
+  // If we havent this schema, read in from config file
+  if ( ! m_schemas.contains( newSchema ) )
+  {
+    // fallback defaults
+    QColor tmp0 (KGlobalSettings::baseColor());
+    QColor tmp1 (KGlobalSettings::highlightColor());
+    QColor tmp2 (KGlobalSettings::alternateBackgroundColor());
+    QColor tmp3 ( "#FFFF99" );
+    QColor tmp4 (tmp2.dark());
+    QColor tmp5 ( KGlobalSettings::textColor() );
+    QColor tmp6 ( "#EAE9E8" );
+    QColor tmp7 ( "#000000" );
 
-  m_back->setColor(config->readColorEntry("Color Background", &tmp0));
-  m_selected->setColor(config->readColorEntry("Color Selection", &tmp1));
-  m_current->setColor(config->readColorEntry("Color Highlighted Line", &tmp2));
-  m_bracket->setColor(config->readColorEntry("Color Highlighted Bracket", &tmp3));
-  m_wwmarker->setColor(config->readColorEntry("Color Word Wrap Marker", &tmp4));
-  m_tmarker->setColor(config->readColorEntry("Color Tab Marker", &tmp5));
-  m_iconborder->setColor(config->readColorEntry("Color Icon Bar", &tmp6));
-  m_linenumber->setColor(config->readColorEntry("Color Line Number", &tmp7));
+    // same std colors like in KateDocument::markColor
+    QValueVector <QColor> mark(KTextEditor::MarkInterface::reservedMarkersCount());
+    Q_ASSERT(mark.size() > 6);
+    mark[0] = Qt::blue;
+    mark[1] = Qt::red;
+    mark[2] = Qt::yellow;
+    mark[3] = Qt::magenta;
+    mark[4] = Qt::gray;
+    mark[5] = Qt::green;
+    mark[6] = Qt::red;
 
-  // same std colors like in KateDocument::markColor
-  QValueVector <QColor> mark(KTextEditor::MarkInterface::reservedMarkersCount());
-  Q_ASSERT(mark.size() > 6);
-  mark[0] = Qt::blue;
-  mark[1] = Qt::red;
-  mark[2] = Qt::yellow;
-  mark[3] = Qt::magenta;
-  mark[4] = Qt::gray;
-  mark[5] = Qt::green;
-  mark[6] = Qt::red;
+    SchemaColors c;
+    KConfig *config = KateFactory::self()->schemaManager()->schema(newSchema);
+
+    c.back= config->readColorEntry("Color Background", &tmp0);
+    c.selected = config->readColorEntry("Color Selection", &tmp1);
+    c.current = config->readColorEntry("Color Highlighted Line", &tmp2);
+    c.bracket = config->readColorEntry("Color Highlighted Bracket", &tmp3);
+    c.wwmarker = config->readColorEntry("Color Word Wrap Marker", &tmp4);
+    c.tmarker = config->readColorEntry("Color Tab Marker", &tmp5);
+    c.iconborder = config->readColorEntry("Color Icon Bar", &tmp6);
+    c.linenumber = config->readColorEntry("Color Line Number", &tmp7);
+
+    for (int i = 0; i < KTextEditor::MarkInterface::reservedMarkersCount(); i++)
+      c.markerColors[i] =  config->readColorEntry( QString("Color MarkType%1").arg(i+1), &mark[i] );
+
+     m_schemas[ newSchema ] = c;
+  }
+
+  m_back->setColor(  m_schemas[ newSchema ].back);
+  m_selected->setColor(  m_schemas [ newSchema ].selected );
+  m_current->setColor(  m_schemas [ newSchema ].current );
+  m_bracket->setColor(  m_schemas [ newSchema ].bracket );
+  m_wwmarker->setColor(  m_schemas [ newSchema ].wwmarker );
+  m_tmarker->setColor(  m_schemas [ newSchema ].tmarker );
+  m_iconborder->setColor(  m_schemas [ newSchema ].iconborder );
+  m_linenumber->setColor(  m_schemas [ newSchema ].linenumber );
 
   // map from 0..reservedMarkersCount()-1 - the same index as in markInterface
   for (int i = 0; i < KTextEditor::MarkInterface::reservedMarkersCount(); i++)
   {
-    // arg(i+1) to match the number conventions in markinterface.h
-    m_markerColors[i] = config->readColorEntry(QString("Color MarkType%1").arg(i + 1), &mark[i]);
     QPixmap pix(16, 16);
-    pix.fill(m_markerColors[i]);
+    pix.fill( m_schemas [ newSchema ].markerColors[i]);
     m_combobox->changeItem(pix, m_combobox->text(i), i);
   }
-  m_markers->setColor( m_markerColors[ m_combobox->currentItem() ] );
+  m_markers->setColor(  m_schemas [ newSchema ].markerColors[ m_combobox->currentItem() ] );
 
   connect( m_back      , SIGNAL( changed( const QColor& ) ), SIGNAL( changed() ) );
   connect( m_selected  , SIGNAL( changed( const QColor& ) ), SIGNAL( changed() ) );
@@ -449,27 +486,37 @@ void KateSchemaConfigColorTab::readConfig (KConfig *config)
   connect( m_markers   , SIGNAL( changed( const QColor& ) ), SLOT( slotMarkerColorChanged( const QColor& ) ) );
 }
 
-void KateSchemaConfigColorTab::writeConfig (KConfig *config)
+void KateSchemaConfigColorTab::apply ()
 {
-  config->writeEntry("Color Background", m_back->color());
-  config->writeEntry("Color Selection", m_selected->color());
-  config->writeEntry("Color Highlighted Line", m_current->color());
-  config->writeEntry("Color Highlighted Bracket", m_bracket->color());
-  config->writeEntry("Color Word Wrap Marker", m_wwmarker->color());
-  config->writeEntry("Color Tab Marker", m_tmarker->color());
-  config->writeEntry("Color Icon Bar", m_iconborder->color());
-  config->writeEntry("Color Line Number", m_linenumber->color());
-
-  for (int i = 0; i < KTextEditor::MarkInterface::reservedMarkersCount(); i++)
+  schemaChanged( m_schema );
+  QMap<int,SchemaColors>::Iterator it;
+  for ( it =  m_schemas.begin(); it !=  m_schemas.end(); ++it )
   {
-    config->writeEntry(QString("Color MarkType%1").arg(i + 1), m_markerColors[i]);
+    kdDebug(13030)<<"APPLY scheme = "<<it.key()<<endl;
+    KConfig *config = KateFactory::self()->schemaManager()->schema( it.key() );
+    kdDebug(13030)<<"Using config group "<<config->group()<<endl;
+    SchemaColors c = it.data();
+
+    config->writeEntry("Color Background", c.back);
+    config->writeEntry("Color Selection", c.selected);
+    config->writeEntry("Color Highlighted Line", c.current);
+    config->writeEntry("Color Highlighted Bracket", c.bracket);
+    config->writeEntry("Color Word Wrap Marker", c.wwmarker);
+    config->writeEntry("Color Tab Marker", c.tmarker);
+    config->writeEntry("Color Icon Bar", c.iconborder);
+    config->writeEntry("Color Line Number", c.linenumber);
+
+    for (int i = 0; i < KTextEditor::MarkInterface::reservedMarkersCount(); i++)
+    {
+      config->writeEntry(QString("Color MarkType%1").arg(i + 1), c.markerColors[i]);
+    }
   }
 }
 
 void KateSchemaConfigColorTab::slotMarkerColorChanged( const QColor& color)
 {
   int index = m_combobox->currentItem();
-  m_markerColors[ index ] = color;
+   m_schemas[ m_schema ].markerColors[ index ] = color;
   QPixmap pix(16, 16);
   pix.fill(color);
   m_combobox->changeItem(pix, m_combobox->text(index), index);
@@ -481,13 +528,13 @@ void KateSchemaConfigColorTab::slotComboBoxChanged(int index)
 {
   // temporarily disconnect the changed-signal because setColor emits changed as well
   m_markers->disconnect( SIGNAL( changed( const QColor& ) ) );
-  m_markers->setColor( m_markerColors[index] );
+  m_markers->setColor( m_schemas[m_schema].markerColors[index] );
   connect( m_markers, SIGNAL( changed( const QColor& ) ), SLOT( slotMarkerColorChanged( const QColor& ) ) );
 }
 
 //END KateSchemaConfigColorTab
 
-//BEGIN FontConfig
+//BEGIN FontConfig -- 'Fonts' tab
 KateSchemaConfigFontTab::KateSchemaConfigFontTab( QWidget *parent, const char * )
   : QWidget (parent)
 {
@@ -499,6 +546,7 @@ KateSchemaConfigFontTab::KateSchemaConfigFontTab( QWidget *parent, const char *
   grid->addWidget( m_fontchooser, 0, 0);
 
   connect (this, SIGNAL( changed()), parent->parentWidget(), SLOT (slotChanged()));
+  m_schema = -1;
 }
 
 KateSchemaConfigFontTab::~KateSchemaConfigFontTab()
@@ -507,29 +555,39 @@ KateSchemaConfigFontTab::~KateSchemaConfigFontTab()
 
 void KateSchemaConfigFontTab::slotFontSelected( const QFont &font )
 {
-  myFont = font;
-
-  emit changed();
+  if ( m_schema > -1 )
+  {
+    m_fonts[m_schema] = font;
+    emit changed();
+  }
 }
 
-void KateSchemaConfigFontTab::readConfig (KConfig *config)
+void KateSchemaConfigFontTab::apply()
 {
+  FontMap::Iterator it;
+  for ( it = m_fonts.begin(); it != m_fonts.end(); ++it )
+  {
+    KateFactory::self()->schemaManager()->schema( it.key() )->writeEntry( "Font", it.data() );
+  }
+}
+
+void KateSchemaConfigFontTab::schemaChanged( int newSchema )
+{
+  if ( m_schema > -1 )
+    m_fonts[ m_schema ] = m_fontchooser->font();
+
+  m_schema = newSchema;
+
   QFont f (KGlobalSettings::fixedFont());
 
   m_fontchooser->disconnect ( this );
-  m_fontchooser->setFont (config->readFontEntry("Font", &f));
-  myFont = m_fontchooser->font();
+  m_fontchooser->setFont ( KateFactory::self()->schemaManager()->schema( newSchema )->readFontEntry("Font", &f) );
+  m_fonts[ newSchema ] = m_fontchooser->font();
   connect (m_fontchooser, SIGNAL (fontSelected( const QFont & )), this, SLOT (slotFontSelected( const QFont & )));
 }
-
-void KateSchemaConfigFontTab::writeConfig (KConfig *config)
-{
-  config->writeEntry("Font", myFont);
-}
-
 //END FontConfig
 
-//BEGIN FontColorConfig
+//BEGIN FontColorConfig -- 'Normal Text Styles' tab
 KateSchemaConfigFontColorTab::KateSchemaConfigFontColorTab( QWidget *parent, const char * )
   : QWidget (parent)
 {
@@ -542,6 +600,14 @@ KateSchemaConfigFontColorTab::KateSchemaConfigFontColorTab( QWidget *parent, con
   grid->addWidget( m_defaultStyles, 0, 0);
 
   connect (m_defaultStyles, SIGNAL (changed()), parent->parentWidget(), SLOT (slotChanged()));
+
+  QWhatsThis::add( m_defaultStyles,  i18n(
+      "This list displays the default styles for the current schema and "
+      "offers the means to edit them. The style name reflects the current "
+      "style settings."
+      "<p>To edit the colors, click the colored squares, or select the color "
+      "to edit from the popup menu.<p>You can unset the Background and Selected "
+      "Background colors from the popup menu when appropriate.") );
 }
 
 KateSchemaConfigFontColorTab::~KateSchemaConfigFontColorTab()
@@ -586,16 +652,6 @@ void KateSchemaConfigFontColorTab::schemaChanged (uint schema)
   {
     new KateStyleListItem( m_defaultStyles, KateHlManager::self()->defaultStyleName(i, true), l->at( i ) );
   }
-
-  QWhatsThis::add( m_defaultStyles,  i18n(
-    "This list displays the default styles for the current schema and "
-    "offers the means to edit them. The style name reflects the current "
-    "style settings.<p>To edit using the keyboard, press "
-    "<strong>&lt;SPACE&gt;</strong> and choose a property from the popup menu."
-    "<p>To edit the colors, click the colored squares, or select the color "
-    "to edit from the popup menu.<p>You can unset the Background and Selected "
-    "Background colors from the popup menu when appropriate.") );
-
 }
 
 void KateSchemaConfigFontColorTab::reload ()
@@ -612,7 +668,7 @@ void KateSchemaConfigFontColorTab::apply ()
 
 //END FontColorConfig
 
-//BEGIN KateSchemaConfigHighlightTab
+//BEGIN KateSchemaConfigHighlightTab -- 'Highlighting Text Styles' tab
 KateSchemaConfigHighlightTab::KateSchemaConfigHighlightTab( QWidget *parent, const char *, KateSchemaConfigFontColorTab *page, uint hl )
   : QWidget (parent)
 {
@@ -678,13 +734,13 @@ void KateSchemaConfigHighlightTab::schemaChanged (uint schema)
 {
   m_schema = schema;
 
-  kdDebug () << "NEW SCHEMA: " << m_schema << " NEW HL: " << m_hl << endl;
+  kdDebug(13030) << "NEW SCHEMA: " << m_schema << " NEW HL: " << m_hl << endl;
 
   m_styles->clear ();
 
   if (!m_hlDict[m_schema])
   {
-    kdDebug () << "NEW SCHEMA, create dict" << endl;
+    kdDebug(13030) << "NEW SCHEMA, create dict" << endl;
 
     m_hlDict.insert (schema, new QIntDict<KateHlItemDataList>);
     m_hlDict[m_schema]->setAutoDelete (true);
@@ -692,7 +748,7 @@ void KateSchemaConfigHighlightTab::schemaChanged (uint schema)
 
   if (!m_hlDict[m_schema]->find(m_hl))
   {
-    kdDebug () << "NEW HL, create list" << endl;
+    kdDebug(13030) << "NEW HL, create list" << endl;
 
     KateHlItemDataList *list = new KateHlItemDataList ();
     KateHlManager::self()->getHl( m_hl )->getKateHlItemDataListCopy (m_schema, *list);
@@ -723,7 +779,7 @@ void KateSchemaConfigHighlightTab::schemaChanged (uint schema)
         itemData != 0L;
         itemData = m_hlDict[m_schema]->find(m_hl)->prev())
   {
-    kdDebug () << "insert items " << itemData->name << endl;
+    kdDebug(13030) << "insert items " << itemData->name << endl;
 
     // All stylenames have their language mode prefixed, e.g. HTML:Comment
     // split them and put them into nice substructures.
@@ -758,12 +814,14 @@ void KateSchemaConfigHighlightTab::apply ()
 {
   for ( QIntDictIterator< QIntDict<KateHlItemDataList> > it( m_hlDict ); it.current(); ++it )
     for ( QIntDictIterator< KateHlItemDataList > it2( *it.current() ); it2.current(); ++it2 )
-       KateHlManager::self()->getHl( it2.currentKey() )->setKateHlItemDataList (it.currentKey(), *(it2.current()));
+    {
+      KateHlManager::self()->getHl( it2.currentKey() )->setKateHlItemDataList (it.currentKey(), *(it2.current()));
+    }
 }
 
 //END KateSchemaConfigHighlightTab
 
-//BEGIN KateSchemaConfigPage
+//BEGIN KateSchemaConfigPage -- Main dialog page
 KateSchemaConfigPage::KateSchemaConfigPage( QWidget *parent, KateDocument *doc )
   : KateConfigPage( parent ),
     m_lastSchema (-1)
@@ -828,21 +886,23 @@ KateSchemaConfigPage::~KateSchemaConfigPage ()
 
 void KateSchemaConfigPage::apply()
 {
-  if (m_lastSchema > -1)
-  {
-    m_colorTab->writeConfig (KateFactory::self()->schemaManager()->schema(m_lastSchema));
-    m_fontTab->writeConfig (KateFactory::self()->schemaManager()->schema(m_lastSchema));
-  }
+  m_colorTab->apply();
+  m_fontTab->apply();
+  m_fontColorTab->apply ();
+  m_highlightTab->apply ();
 
   // just sync the config
   KateFactory::self()->schemaManager()->schema (0)->sync();
+
   KateFactory::self()->schemaManager()->update ();
 
-  KateRendererConfig::global()->setSchema (defaultSchemaCombo->currentItem());
+  // clear all attributes
+  for (int i = 0; i < KateHlManager::self()->highlights(); ++i)
+    KateHlManager::self()->getHl (i)->clearAttributeArrays ();
 
-  // special for the highlighting stuff
-  m_fontColorTab->apply ();
-  m_highlightTab->apply ();
+  // than reload the whole stuff
+  KateRendererConfig::global()->setSchema (defaultSchemaCombo->currentItem());
+  KateRendererConfig::global()->reloadSchema();
 
   // sync the hl config for real
   KateHlManager::self()->getKConfig()->sync ();
@@ -921,23 +981,10 @@ void KateSchemaConfigPage::newSchema ()
 
 void KateSchemaConfigPage::schemaChanged (int schema)
 {
-  if (schema < 2)
-  {
-    btndel->setEnabled (false);
-  }
-  else
-  {
-    btndel->setEnabled (true);
-  }
+  btndel->setEnabled( schema > 1 );
 
-  if (m_lastSchema > -1) // so, pressing "Cancel" is no good here?
-  {
-    m_colorTab->writeConfig (KateFactory::self()->schemaManager()->schema(m_lastSchema));
-    m_fontTab->writeConfig (KateFactory::self()->schemaManager()->schema(m_lastSchema));
-  }
-
-  m_colorTab->readConfig (KateFactory::self()->schemaManager()->schema(schema));
-  m_fontTab->readConfig (KateFactory::self()->schemaManager()->schema(schema));
+  m_colorTab->schemaChanged( schema );
+  m_fontTab->schemaChanged( schema );
   m_fontColorTab->schemaChanged (schema);
   m_highlightTab->schemaChanged (schema);
 
@@ -951,7 +998,7 @@ void KateSchemaConfigPage::newCurrentPage (QWidget *w)
 }
 //END KateSchemaConfigPage
 
-// BEGIN SCHEMA ACTION
+//BEGIN SCHEMA ACTION -- the 'View->Schema' menu action
 void KateViewSchemaAction::init()
 {
   m_view = 0;
@@ -996,7 +1043,7 @@ void KateViewSchemaAction::setSchema (int mode)
   if (view)
     view->renderer()->config()->setSchema (mode-1);
 }
-// END SCHEMA ACTION
+//END SCHEMA ACTION
 
 //BEGIN KateStyleListView
 KateStyleListView::KateStyleListView( QWidget *parent, bool showUseDefaults )
@@ -1016,8 +1063,8 @@ KateStyleListView::KateStyleListView( QWidget *parent, bool showUseDefaults )
     addColumn( i18n("Use Default Style") );
   connect( this, SIGNAL(mouseButtonPressed(int, QListViewItem*, const QPoint&, int)),
            this, SLOT(slotMousePressed(int, QListViewItem*, const QPoint&, int)) );
-  connect( this, SIGNAL(spacePressed(QListViewItem*)),
-           this, SLOT(showPopupMenu(QListViewItem*)) );
+  connect( this, SIGNAL(contextMenuRequested(QListViewItem*,const QPoint&, int)),
+           this, SLOT(showPopupMenu(QListViewItem*, const QPoint&)) );
   // grap the bg color, selected color and default font
   normalcol = KGlobalSettings::textColor();
   bgcol = KateRendererConfig::global()->backgroundColor();
@@ -1085,10 +1132,10 @@ void KateStyleListView::showPopupMenu( KateStyleListItem *i, const QPoint &globa
   m.exec( globalPos );
 }
 
-void KateStyleListView::showPopupMenu( QListViewItem *i )
+void KateStyleListView::showPopupMenu( QListViewItem *i, const QPoint &pos )
 {
   if ( dynamic_cast<KateStyleListItem*>(i) )
-    showPopupMenu( (KateStyleListItem*)i, viewport()->mapToGlobal(itemRect(i).topLeft()), true );
+    showPopupMenu( (KateStyleListItem*)i, pos, true );
 }
 
 void KateStyleListView::mSlotPopupHandler( int z )
@@ -1099,6 +1146,7 @@ void KateStyleListView::mSlotPopupHandler( int z )
 void KateStyleListView::unsetColor( int c )
 {
   ((KateStyleListItem*)currentItem())->unsetColor( c );
+  emitChanged();
 }
 
 // Because QListViewItem::activatePos() is going to become deprecated,
@@ -1106,10 +1154,7 @@ void KateStyleListView::unsetColor( int c )
 void KateStyleListView::slotMousePressed(int btn, QListViewItem* i, const QPoint& pos, int c)
 {
   if ( dynamic_cast<KateStyleListItem*>(i) ) {
-    if ( btn == Qt::RightButton ) {
-      showPopupMenu( (KateStyleListItem*)i, /*mapToGlobal(*/pos/*)*/ );
-    }
-    else if ( btn == Qt::LeftButton && c > 0 ) {
+     if ( btn == Qt::LeftButton && c > 0 ) {
       // map pos to item/column and call KateStyleListItem::activate(col, pos)
       ((KateStyleListItem*)i)->activate( c, viewport()->mapFromGlobal( pos ) - QPoint( 0, itemRect(i).top() ) );
     }
@@ -1164,12 +1209,14 @@ void KateStyleListItem::updateStyle()
     if ( is->weight() != st->weight())
       st->setWeight( is->weight() );
   }
+  else st->clearAttribute( KateAttribute::Weight );
 
   if ( is->itemSet(KateAttribute::Italic) )
   {
     if ( is->italic() != st->italic())
       st->setItalic( is->italic() );
   }
+  else st->clearAttribute( KateAttribute::Italic );
 
   if ( is->itemSet(KateAttribute::StrikeOut) )
   {
@@ -1177,42 +1224,49 @@ void KateStyleListItem::updateStyle()
 
       st->setStrikeOut( is->strikeOut() );
   }
+  else st->clearAttribute( KateAttribute::StrikeOut );
 
   if ( is->itemSet(KateAttribute::Underline) )
   {
     if ( is->underline() != st->underline())
       st->setUnderline( is->underline() );
   }
+  else st->clearAttribute( KateAttribute::Underline );
 
   if ( is->itemSet(KateAttribute::Outline) )
   {
     if ( is->outline() != st->outline())
       st->setOutline( is->outline() );
   }
+  else st->clearAttribute( KateAttribute::Outline );
 
   if ( is->itemSet(KateAttribute::TextColor) )
   {
     if ( is->textColor() != st->textColor())
       st->setTextColor( is->textColor() );
   }
+  else st->clearAttribute( KateAttribute::TextColor );
 
   if ( is->itemSet(KateAttribute::SelectedTextColor) )
   {
     if ( is->selectedTextColor() != st->selectedTextColor())
       st->setSelectedTextColor( is->selectedTextColor() );
   }
+  else st->clearAttribute( KateAttribute::SelectedTextColor);
 
   if ( is->itemSet(KateAttribute::BGColor) )
   {
     if ( is->bgColor() != st->bgColor())
       st->setBGColor( is->bgColor() );
   }
+  else st->clearAttribute( KateAttribute::BGColor );
 
   if ( is->itemSet(KateAttribute::SelectedBGColor) )
   {
     if ( is->selectedBGColor() != st->selectedBGColor())
       st->setSelectedBGColor( is->selectedBGColor() );
   }
+  else st->clearAttribute( KateAttribute::SelectedBGColor );
 }
 
 /* only true for a hl mode item using it's default style */
@@ -1302,6 +1356,7 @@ void KateStyleListItem::toggleDefStyle()
   else {
     delete is;
     is = new KateAttribute( *ds );
+    updateStyle();
     repaint();
   }
 }
