@@ -1,6 +1,6 @@
 /*
 
-   $Id: kprocess.cpp 348106 2004-09-21 09:47:32Z ossi $
+   $Id: kprocess.cpp 503447 2006-01-29 09:42:40Z ossi $
 
    This file is part of the KDE libraries
    Copyright (C) 1997 Christian Czezatke (e9025461@student.tuwien.ac.at)
@@ -17,8 +17,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 
@@ -1005,6 +1005,10 @@ int KProcess::commSetupDoneC()
     if (dup2(out[1], STDOUT_FILENO) < 0 ||
         setsockopt(out[1], SOL_SOCKET, SO_LINGER, (char *)&so, sizeof(so)))
       ok = 0;
+    if (communication & MergedStderr) {
+      if (dup2(out[1], STDERR_FILENO) < 0)
+        ok = 0;
+    }
   }
   if (d->usePty & Stderr) {
     if (dup2(d->pty->slaveFd(), STDERR_FILENO) < 0) ok = 0;
