@@ -2,7 +2,7 @@
  * This file is part of the DOM implementation for KDE.
  *
  * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
- *           (C) 2002 Apple Computer, Inc.
+ *           (C) 2004 Apple Computer, Inc.
  *           (C) 2005 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include "dom/css_value.h"
@@ -52,6 +52,17 @@ using khtml::FontDef;
 
 using namespace DOM;
 
+// Quotes the string if it needs quoting.
+static DOMString quoteStringIfNeeded(const DOMString &string)
+{
+    // FIXME: Also need to transform control characters into \ sequences.
+    QString s = string.string();
+    s.replace('\\', "\\\\");
+    s.replace('\'', "\\'");
+    return '\'' + s + '\'';
+}
+
+
 CSSStyleDeclarationImpl::CSSStyleDeclarationImpl(CSSRuleImpl *parent)
     : StyleBaseImpl(parent)
 {
@@ -528,15 +539,9 @@ void CSSPrimitiveValueImpl::cleanup()
 int CSSPrimitiveValueImpl::computeLength( khtml::RenderStyle *style, QPaintDeviceMetrics *devMetrics )
 {
     double result = computeLengthFloat( style, devMetrics );
-    int intResult = (int)result;
-#ifdef APPLE_CHANGES
-    // This conversion is imprecise, often resulting in values of e.g., 44.99998.  We
+    // This conversion is imprecise, often resulting in values of, e.g., 44.99998.  We
     // need to go ahead and round if we're really close to the next integer value.
-    double newResult = (intResult < 0) ? result-0.01 : result+0.01;
-    int secondIntResult = (int)newResult;
-    if (secondIntResult != intResult)
-        return secondIntResult;
-#endif
+    int intResult = (int)(result + (result < 0 ? -0.01 : +0.01));
     return intResult;
 }
 
@@ -576,14 +581,14 @@ double CSSPrimitiveValueImpl::computeLengthFloat( khtml::RenderStyle *style, QPa
 	factor = dpiY/25.4;
         break;
     case CSSPrimitiveValue::CSS_IN:
-            factor = dpiY;
+        factor = dpiY;
         break;
     case CSSPrimitiveValue::CSS_PT:
-            factor = dpiY/72.;
+        factor = dpiY/72.;
         break;
     case CSSPrimitiveValue::CSS_PC:
         // 1 pc == 12 pt
-            factor = dpiY*12./72.;
+        factor = dpiY*12./72.;
         break;
     default:
         return -1;
@@ -707,7 +712,7 @@ DOM::DOMString CSSPrimitiveValueImpl::cssText() const
 	    // ###
 	    break;
 	case CSSPrimitiveValue::CSS_STRING:
-	    // ###
+	    text = quoteStringIfNeeded(m_value.string);
 	    break;
 	case CSSPrimitiveValue::CSS_URI:
             text  = "url(";
@@ -945,7 +950,7 @@ QString QuotesValueImpl::openQuote(int level) const
 //     kdDebug( 6080 ) << "Open quote level:" << level << endl;
     if (level < 0) level = 0;
     else
-    if (level >= levels) level = levels-1;
+    if (level >= (int) levels) level = (int) (levels-1);
     return data[level*2];
 }
 
@@ -955,7 +960,7 @@ QString QuotesValueImpl::closeQuote(int level) const
 //     kdDebug( 6080 ) << "Close quote level:" << level << endl;
     if (level < 0) level = 0;
     else
-    if (level >= levels) level = levels-1;
+    if (level >= (int) levels) level = (int) (levels-1);
     return data[level*2+1];
 }
 
