@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 // -------------------------------------------------------------------------
 
@@ -140,23 +140,35 @@ void HTMLBodyElementImpl::parseAttribute(AttributeImpl *attr)
     }
     case ATTR_ONLOAD:
         getDocument()->setHTMLWindowEventListener(EventImpl::LOAD_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(),"onload"));
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onload", NULL));
         break;
     case ATTR_ONUNLOAD:
         getDocument()->setHTMLWindowEventListener(EventImpl::UNLOAD_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(),"onunload"));
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onunload", NULL));
         break;
     case ATTR_ONBLUR:
         getDocument()->setHTMLWindowEventListener(EventImpl::BLUR_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(),"onblur"));
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onblur", NULL));
         break;
     case ATTR_ONFOCUS:
         getDocument()->setHTMLWindowEventListener(EventImpl::FOCUS_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(),"onfocus"));
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onfocus", NULL));
         break;
     case ATTR_ONRESIZE:
         getDocument()->setHTMLWindowEventListener(EventImpl::RESIZE_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(),"onresize"));
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onresize", NULL));
+        break;
+    case ATTR_ONKEYUP:
+        getDocument()->setHTMLWindowEventListener(EventImpl::KEYUP_EVENT,
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onkeyup", NULL));
+        break;
+    case ATTR_ONKEYDOWN:
+        getDocument()->setHTMLWindowEventListener(EventImpl::KEYDOWN_EVENT,
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onkeydown", NULL));
+        break;
+    case ATTR_ONKEYPRESS:
+        getDocument()->setHTMLWindowEventListener(EventImpl::KEYPRESS_EVENT,
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onkeypress", NULL));
         break;
     case ATTR_NOSAVE:
 	break;
@@ -206,7 +218,11 @@ void HTMLBodyElementImpl::attach()
     RenderStyle* style = getDocument()->styleSelector()->styleForElement(this);
     style->ref();
     if (parentNode()->renderer() && style->display() != NONE) {
-        m_render = new (getDocument()->renderArena()) RenderBody(this);
+        if (style->display() == BLOCK)
+            // only use the quirky class for block display
+            m_render = new (getDocument()->renderArena()) RenderBody(this);
+        else
+            m_render = RenderObject::createObject(this, style);
         m_render->setStyle(style);
         parentNode()->renderer()->addChild(m_render, nextRenderer());
     }
@@ -245,12 +261,6 @@ void HTMLFrameElementImpl::parseAttribute(AttributeImpl *attr)
     case ATTR_SRC:
         setLocation(khtml::parseURL(attr->val()));
         break;
-    case ATTR_ID:
-    case ATTR_NAME:
-        // FIXME: if already attached, doesn't change the frame name
-        // FIXME: frame name conflicts, no unique frame name anymore
-        name = attr->value();
-        break;
     case ATTR_FRAMEBORDER:
     {
         frameBorder = attr->value().toInt();
@@ -281,13 +291,18 @@ void HTMLFrameElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ONLOAD:
         setHTMLEventListener(EventImpl::LOAD_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string(),"onload"));
+            getDocument()->createHTMLEventListener(attr->value().string(), "onload", this));
         break;
     case ATTR_ONUNLOAD:
         setHTMLEventListener(EventImpl::UNLOAD_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string(),"onunload"));
+            getDocument()->createHTMLEventListener(attr->value().string(), "onunload", this));
         break;
-
+    case ATTR_ID:
+    case ATTR_NAME:
+        // FIXME: if already attached, doesn't change the frame name
+        // FIXME: frame name conflicts, no unique frame name anymore
+        name = attr->value();
+        //fallthrough intentional, let the base handle it
     default:
         HTMLElementImpl::parseAttribute(attr);
     }
@@ -298,8 +313,6 @@ void HTMLFrameElementImpl::attach()
     assert(!attached());
     assert(parentNode());
 
-    // we should first look up via id, then via name.
-    // this shortterm hack fixes the ugly case. ### rewrite needed for next release
     name = getAttribute(ATTR_NAME);
     if (name.isNull())
         name = getAttribute(ATTR_ID);
@@ -347,8 +360,6 @@ void HTMLFrameElementImpl::attach()
 
 void HTMLFrameElementImpl::setLocation( const DOMString& str )
 {
-    if ( url == str )
-        return;
 
     url = str;
 
@@ -405,6 +416,18 @@ DocumentImpl* HTMLFrameElementImpl::contentDocument() const
     return 0;
 }
 
+KHTMLPart*   HTMLFrameElementImpl::contentPart() const
+{
+    if ( !m_render ) return 0;
+
+    RenderPart* render = static_cast<RenderPart*>( m_render );
+
+    if(render->widget() && ::qt_cast<KHTMLView*>( render->widget()) )
+        return static_cast<KHTMLView*>( render->widget() )->part();
+
+    return 0;
+}
+
 // -------------------------------------------------------------------------
 
 HTMLFrameSetElementImpl::HTMLFrameSetElementImpl(DocumentPtr *doc)
@@ -469,11 +492,11 @@ void HTMLFrameSetElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ONLOAD:
         getDocument()->setHTMLEventListener(EventImpl::LOAD_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(),"onload"));
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onload", this));
         break;
     case ATTR_ONUNLOAD:
         getDocument()->setHTMLEventListener(EventImpl::UNLOAD_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(),"onunload"));
+	    getDocument()->createHTMLEventListener(attr->value().string(), "onunload", this));
         break;
     default:
         HTMLElementImpl::parseAttribute(attr);
@@ -603,6 +626,10 @@ void HTMLIFrameElementImpl::attach()
     assert(!m_render);
     assert(parentNode());
 
+    name = getAttribute(ATTR_NAME);
+    if (name.isNull())
+        name = getAttribute(ATTR_ID);
+
     RenderStyle* style = getDocument()->styleSelector()->styleForElement(this);
     style->ref();
     if (getDocument()->isURLAllowed(url.string()) &&
@@ -621,16 +648,16 @@ void HTMLIFrameElementImpl::attach()
         if(w && (name.isEmpty() || w->part()->frameExists( name.string() )))
             name = DOMString(w->part()->requestFrameName());
 
-        static_cast<RenderPartObject*>(m_render)->updateWidget();
         needWidgetUpdate = false;
+        static_cast<RenderPartObject*>(m_render)->updateWidget();
     }
 }
 
 void HTMLIFrameElementImpl::recalcStyle( StyleChange ch )
 {
     if (needWidgetUpdate) {
-        if(m_render)  static_cast<RenderPartObject*>(m_render)->updateWidget();
         needWidgetUpdate = false;
+        if(m_render)  static_cast<RenderPartObject*>(m_render)->updateWidget();
     }
     HTMLElementImpl::recalcStyle( ch );
 }
