@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 #ifndef render_object_h
@@ -82,6 +82,20 @@ typedef enum {
     HitTestChildrenOnly = 2
 } HitTestAction;
 
+typedef enum {
+    PageBreakNormal = 0, // all rules apply
+    PageBreakHarder = 1, // page-break-inside: avoid is ignored
+    PageBreakForced = 2  // page-break-after/before: avoid, orphans and widows ignored
+} PageBreakLevel;
+
+inline PageBreakLevel operator| (PageBreakLevel a, PageBreakLevel b) {
+    if (a == PageBreakForced || b == PageBreakForced)
+        return PageBreakForced;
+    if (a == PageBreakHarder || b == PageBreakHarder)
+        return PageBreakHarder;
+    return PageBreakNormal;
+}
+
 namespace DOM {
     class HTMLAreaElementImpl;
     class DOMString;
@@ -131,6 +145,7 @@ public:
 
     virtual RenderLayer* layer() const { return 0; }
     RenderLayer* enclosingLayer() const;
+    RenderLayer* enclosingStackingContext() const;
     void addLayers(RenderLayer* parentLayer, RenderObject* newObject);
     void removeLayers(RenderLayer* parentLayer);
     void moveLayers(RenderLayer* oldParent, RenderLayer* newParent);
@@ -161,17 +176,17 @@ public:
     RenderObject *objectAbove() const;
 
     // Returns if an object has counter-increment or counter-reset
-    bool hasCounter(const DOM::DOMString& counter) const;
+    bool hasCounter(const QString& counter) const;
     // Calculates the value of the counter
-    CounterNode* getCounter(const DOM::DOMString& counter, bool view = false, bool counters = false);
+    CounterNode* getCounter(const QString& counter, bool view = false, bool counters = false);
     // Detaches all counterNodes
     void detachCounters();
 
 
 protected:
     // Helper functions for counter-cache
-    void insertCounter(const DOM::DOMString& counter, CounterNode* value);
-    CounterNode* lookupCounter(const DOM::DOMString& counter) const;
+    void insertCounter(const QString& counter, CounterNode* value);
+    CounterNode* lookupCounter(const QString& counter) const;
 
 public:
     //////////////////////////////////////////
@@ -219,7 +234,8 @@ public:
     virtual bool isInlineContinuation() const { return false; }
 
 
-    bool isRoot() const;
+    bool isRoot() const { return m_isRoot && !m_isAnonymous; }
+    void setIsRoot(bool b) { m_isRoot = b; }
     bool isHR() const;
     // some helper functions...
     virtual bool isRenderBlock() const { return false; }
@@ -231,6 +247,8 @@ public:
     virtual bool isBox() const { return false; }
     virtual bool isRenderReplaced() const { return false; }
 
+    virtual bool isCounter() const { return false; }
+    virtual bool isQuote() const { return false; }
     virtual bool isListItem() const { return false; }
     virtual bool isListMarker() const { return false; }
     virtual bool isCanvas() const { return false; }
@@ -259,6 +277,7 @@ public:
     bool isText() const { return m_isText; }
     bool isInline() const { return m_inline; }
     bool isCompact() const { return style()->display() == COMPACT; } // compact
+    bool isRunIn() const { return style()->display() == RUN_IN; } // run-in object
     bool mouseInside() const;
     bool isReplaced() const { return m_replaced; }
     bool isReplacedBlock() const { return isInline() && isReplaced() && isRenderBlock(); }
@@ -321,6 +340,8 @@ public:
 
     void scheduleRelayout(RenderObject *clippedObj = 0);
 
+    void updateBackgroundImages(RenderStyle* oldStyle);
+
     virtual InlineBox* createInlineBox(bool makePlaceHolderBox, bool isRootLineBox);
 
     virtual short lineHeight( bool firstLine ) const;
@@ -346,7 +367,7 @@ public:
 
     virtual void paintBoxDecorations(PaintInfo&, int /*_tx*/, int /*_ty*/) {}
 
-    virtual void paintBackgroundExtended(QPainter* /*p*/, const QColor& /*c*/, CachedImage* /*bg*/,
+    virtual void paintBackgroundExtended(QPainter* /*p*/, const QColor& /*c*/, const BackgroundLayer */*bgLayer*/,
                                          int /*clipy*/, int /*cliph*/, int /*_tx*/, int /*_ty*/,
                                          int /*w*/, int /*height*/, int /*bleft*/, int /*bright*/ ) {}
 
@@ -405,6 +426,35 @@ public:
     // of a containing block).  HTML4 buttons, legends, and floating/compact elements do this.
     bool sizesToMaxWidth() const;
 
+    /*
+     * NeesPageClear indicates the object crossed a page-break but could not break itself and now
+     * needs to be moved clear by its parent.
+     */
+    void setNeedsPageClear(bool b = true) { m_needsPageClear = b; }
+    virtual bool needsPageClear() const { return m_needsPageClear; }
+
+    /*
+     * ContainsPageBreak indicates the object contains a clean page-break.
+     * ### should be removed and replaced with (crossesPageBreak && !needsPageClear)
+     */
+    void setContainsPageBreak(bool b = true) { m_containsPageBreak = b; }
+    virtual bool containsPageBreak() const { return m_containsPageBreak; }
+
+    virtual int pageTopAfter(int y) const { if (parent()) return parent()->pageTopAfter(y); else return 0; }
+
+    virtual int crossesPageBreak(int top, int bottom) const
+    { if (parent()) return parent()->crossesPageBreak(top, bottom); else return 0; }
+
+    // Checks if a page-break before child is possible at the given page-break level
+    // false means the child should attempt the break self.
+    virtual bool canClear(RenderObject */*child*/, PageBreakLevel level)
+    { if (parent()) return parent()->canClear(this, level); else return false; }
+
+    void setAfterPageBreak(bool b = true)  { m_afterPageBreak = b; };
+    void setBeforePageBreak(bool b = true) { m_beforePageBreak = b; };
+    virtual bool afterPageBreak() const  { return m_afterPageBreak; }
+    virtual bool beforePageBreak() const { return m_beforePageBreak; }
+
     // does a query on the rendertree and finds the innernode
     // and overURL for the given position
     // if readonly == false, it will recalc hover styles accordingly
@@ -488,6 +538,11 @@ public:
     virtual int xPos() const { return 0; }
     virtual int yPos() const { return 0; }
 
+    /** the position of the object from where it begins drawing, including
+     * its negative overflow
+     */
+    int effectiveXPos() const { return xPos() - (hasOverflowClip() ? 0 : negativeOverflowWidth()); }
+
     /** Leftmost coordinate of this inline element relative to containing
      * block. Always zero for non-inline elements.
      */
@@ -510,6 +565,8 @@ public:
     // of borderTop() + paddingTop() + 100px.
     virtual int overflowHeight() const { return height(); }
     virtual int overflowWidth() const { return width(); }
+    // how much goes over the left hand side (0 or a positive number)
+    virtual int negativeOverflowWidth() const { return 0; }
 
     /**
      * Returns the height that is effectively considered when contemplating the
@@ -520,7 +577,7 @@ public:
      * Returns the width that is effectively considered when contemplating the
      * object as a whole -- usually the overflow width, or the width if clipped.
      */
-    int effectiveWidth() const { return hasOverflowClip() ? width() : overflowWidth(); }
+    int effectiveWidth() const { return hasOverflowClip() ? width() : overflowWidth() + negativeOverflowWidth(); }
 
     // IE extensions, heavily used in ECMA
     virtual short offsetWidth() const { return width(); }
@@ -560,7 +617,7 @@ public:
     virtual int borderRight() const { return style()->borderRightWidth(); }
 
     virtual short minWidth() const { return 0; }
-    virtual short maxWidth() const { return 0; }
+    virtual int maxWidth() const { return 0; }
 
     RenderStyle* style() const { return m_style; }
     RenderStyle* style( bool firstLine ) const {
@@ -683,6 +740,8 @@ public:
     virtual long maxOffset() const { return 0; }
 
     virtual void setPixmap(const QPixmap &, const QRect&, CachedImage *);
+    
+    QRegion visibleFlowRegion(int x, int y) const;
 
 protected:
     virtual void selectionStartEnd(int& spos, int& epos);
@@ -690,7 +749,7 @@ protected:
     virtual QRect viewRect() const;
     void remove();
     void invalidateVerticalPositions();
-
+    void updateWidgetMasks();
 
     virtual void removeLeftoverAnonymousBoxes();
 
@@ -703,12 +762,6 @@ private:
     RenderObject *m_previous;
     RenderObject *m_next;
 
-    struct CounterList {
-        DOM::DOMString counter;
-        CounterNode *m_counterNode;
-        CounterList *next;
-    } *m_counterList;
-
     short m_verticalPosition;
 
     bool m_needsLayout               : 1;
@@ -734,7 +787,16 @@ private:
     bool m_mouseInside               : 1;
     bool m_hasFirstLine              : 1;
     bool m_isSelectionBorder         : 1;
-    // note: do not add unnecessary bitflags, we have 32 bit already!
+
+    bool m_isRoot                    : 1;
+
+    bool m_beforePageBreak           : 1;
+    bool m_afterPageBreak            : 1;
+
+    bool m_needsPageClear            : 1;
+    bool m_containsPageBreak         : 1;
+
+    // ### we have 16 + 23 bits. Cut 4 and save 32
 
 
     void arenaDelete(RenderArena *arena, void *objectBase);
