@@ -4,6 +4,7 @@
  * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
  *           (C) 2003 Apple Computer, Inc.
+ *           (C) 2004-2005 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -36,12 +37,16 @@
 #include "xml/dom_nodeimpl.h"
 
 #include "misc/loader.h"
+#include "misc/helper.h"
 
+#include <qbitmap.h>
+#include <qimage.h>
 #include <qpainter.h>
 #include <kdebug.h>
 #include <kglobal.h>
 #include <assert.h>
 #include <limits.h>
+#include <math.h>
 
 #ifdef HAVE_ALLOCA_H
 // explicitly included for systems that don't provide it in stdlib.h
@@ -59,6 +64,9 @@ static bool inInlineTextBoxDetach;
 
 void InlineTextBox::detach(RenderArena* renderArena)
 {
+    if (m_parent)
+        m_parent->removeFromLine(this);
+
 #ifndef NDEBUG
     inInlineTextBoxDetach = true;
 #endif
@@ -84,75 +92,6 @@ void InlineTextBox::operator delete(void* ptr, size_t sz)
     *(size_t *)ptr = sz;
 }
 
-/** checks whether the given colors have enough contrast
- * @returns @p true if contrast is ok.
- */
-inline bool hasSufficientContrast(const QColor &c1, const QColor &c2)
-{
-// New version from Germain Garand, better suited for contrast measurement
-#if 1
-
-#define HUE_DISTANCE 40
-#define CONTRAST_DISTANCE 10
-
-  int h1, s1, v1, h2, s2, v2;
-  int hdist = -CONTRAST_DISTANCE;
-  c1.hsv(&h1,&s1,&v1);
-  c2.hsv(&h2,&s2,&v2);
-  if(h1!=-1 && h2!=-1) { // grey values have no hue
-      hdist = kAbs(h1-h2);
-      if (hdist > 180) hdist = 360-hdist;
-      if (hdist < HUE_DISTANCE) {
-          hdist -= HUE_DISTANCE;
-          // see if they are high key or low key colours
-          bool hk1 = h1>=45 && h1<=225;
-          bool hk2 = h2>=45 && h2<=225;
-          if (hk1 && hk2)
-              hdist = (5*hdist)/3;
-          else if (!hk1 && !hk2)
-              hdist = (7*hdist)/4;
-      }
-      hdist = kMin(hdist, HUE_DISTANCE*2);
-  }
-  return hdist + (kAbs(s1-s2)*128)/(160+kMin(s1,s2)) + kAbs(v1-v2) > CONTRAST_DISTANCE;
-
-#undef CONTRAST_DISTANCE
-#undef HUE_DISTANCE
-
-#else	// orginal fast but primitive version by me (LS)
-
-// ### arbitrary value, to be adapted if necessary (LS)
-#define CONTRAST_DISTANCE 32
-
-  if (kAbs(c1.red() - c2.red()) > CONTRAST_DISTANCE) return true;
-  if (kAbs(c1.green() - c2.green()) > CONTRAST_DISTANCE) return true;
-  if (kAbs(c1.blue() - c2.blue()) > CONTRAST_DISTANCE) return true;
-
-  return false;
-
-#undef CONTRAST_DISTANCE
-
-#endif
-}
-
-/** finds out the background color of an element
- * @param obj render object
- * @return the background color. It is guaranteed that a valid color is returned.
- */
-inline QColor retrieveBackgroundColor(const RenderObject *obj)
-{
-  QColor result;
-  while (!obj->isCanvas()) {
-    result = obj->style()->backgroundColor();
-    if (result.isValid()) return result;
-
-    obj = obj->containingBlock();
-  }/*wend*/
-
-  // everything transparent? Use base then.
-  return obj->style()->palette().active().base();
-}
-
 /** returns the proper ::selection pseudo style for the given element
  * @return the style or 0 if no ::selection pseudo applies.
  */
@@ -189,10 +128,10 @@ void InlineTextBox::paintSelection(const Font *f, RenderText *text, QPainter *p,
         hc = grp.highlightedText();
         hbg = grp.highlight();
 	// ### should be at most retrieved once per render text
-	QColor bg = retrieveBackgroundColor(text);
+	QColor bg = khtml::retrieveBackgroundColor(text);
 	// It may happen that the contrast is -- well -- virtually non existent.
 	// In this case, simply invert the colors
-	if (!hasSufficientContrast(hbg, bg)) {
+	if (!khtml::hasSufficientContrast(hbg, bg)) {
 	    hc = QColor(0xff-hc.red(),0xff-hc.green(),0xff-hc.blue());
 	    hbg = QColor(0xff-hbg.red(),0xff-hbg.green(),0xff-hbg.blue());
 	}/*end if*/
@@ -200,38 +139,13 @@ void InlineTextBox::paintSelection(const Font *f, RenderText *text, QPainter *p,
 
     p->setPen(hc);
 
-    //kdDebug( 6040 ) << "textRun::painting(" << QConstString(text->str->s + m_start, m_len).string() << ") at(" << m_x+tx << "/" << m_y+ty << ")" << endl;
+    //kdDebug( 6040 ) << "textRun::painting(" << QConstString(text->str->s + m_start, m_len).string().left(30) << ") at(" << m_x+tx << "/" << m_y+ty << ")" << endl;
     f->drawText(p, m_x + tx, m_y + ty + m_baseline, text->str->s, text->str->l,
     		m_start, m_len, m_toAdd,
 		m_reversed ? QPainter::RTL : QPainter::LTR,
-		startPos, endPos, hbg, m_y + ty, text->lineHeight(m_firstLine), deco);
+		startPos, endPos, hbg, m_y + ty, height(), deco);
 }
 
-#ifdef APPLE_CHANGES
-void InlineTextBox::paintDecoration( QPainter *pt, int _tx, int _ty, int deco)
-{
-    _tx += m_x;
-    _ty += m_y;
-
-    // Get the text decoration colors.
-    QColor underline, overline, linethrough;
-    object()->getTextDecorationColors(deco, underline, overline, linethrough, true);
-
-    // Use a special function for underlines to get the positioning exactly right.
-    if (deco & UNDERLINE) {
-        pt->setPen(underline);
-        pt->drawLineForText(_tx, _ty, m_baseline, m_width);
-    }
-    if (deco & OVERLINE) {
-        pt->setPen(overline);
-        pt->drawLineForText(_tx, _ty, 0, m_width);
-    }
-    if (deco & LINE_THROUGH) {
-        pt->setPen(linethrough);
-        pt->drawLineForText(_tx, _ty, 2*m_baseline/3, m_width);
-    }
-}
-#else
 void InlineTextBox::paintDecoration( QPainter *pt, const Font *f, int _tx, int _ty, int deco)
 {
     _tx += m_x;
@@ -259,7 +173,105 @@ void InlineTextBox::paintDecoration( QPainter *pt, const Font *f, int _tx, int _
     // NO! Do NOT add BLINK! It is the most annouing feature of Netscape, and IE has a reason not to
     // support it. Lars
 }
-#endif
+
+void InlineTextBox::paintShadow(QPainter *pt, const Font *f, int _tx, int _ty, const ShadowData *shadow )
+{
+    int x = m_x + _tx + shadow->x;
+    int y = m_y + _ty + shadow->y;
+    const RenderText* text = renderText();
+
+    if (shadow->blur <= 0) {
+        QColor c = pt->pen().color();
+        pt->setPen(shadow->color);
+        f->drawText(pt, x, y+m_baseline, text->str->s, text->str->l,
+                    m_start, m_len, m_toAdd,
+                    m_reversed ? QPainter::RTL : QPainter::LTR);
+        pt->setPen(c);
+
+    }
+    else {
+        const int thickness = shadow->blur;
+        const int w = m_width+2*thickness;
+        const int h = m_height+2*thickness;
+        const QRgb color = shadow->color.rgb();
+        const int gray = qGray(color);
+        const bool inverse = (gray < 100);
+        const QRgb bgColor = (inverse) ? qRgb(255,255,255) : qRgb(0,0,0);
+        QPixmap pixmap(w, h);
+        pixmap.fill(bgColor);
+        QPainter p;
+
+        p.begin(&pixmap);
+        p.setPen(shadow->color);
+        p.setFont(pt->font());
+        f->drawText(&p, thickness, thickness+m_baseline, text->str->s, text->str->l,
+                    m_start, m_len, m_toAdd,
+                    m_reversed ? QPainter::RTL : QPainter::LTR);
+
+        p.end();
+        QImage img = pixmap.convertToImage().convertDepth(32);
+
+        int md = thickness*thickness; // max-dist^2
+
+        // blur map (division cache)
+        float *bmap = (float*)alloca(sizeof(float)*(md+1));
+        for(int n=0; n<=md; n++) {
+            float f;
+            f = n/(float)(md+1);
+            f = 1.0 - f*f;
+            bmap[n] = f;
+        }
+
+        float factor = 0.0; // maximal opacity-sum
+        for(int n=-thickness; n<=thickness; n++)
+            for(int m=-thickness; m<=thickness; m++) {
+                int d = n*n+m*m;
+                if (d<=md)
+                    factor += bmap[d];
+            }
+
+        factor = 1.0/factor;
+
+        // alpha map
+        float* amap = (float*)alloca(sizeof(float)*(h*w));
+        memset(amap, 0, h*w*(sizeof(float)));
+        for(int j=thickness; j<h-thickness; j++) {
+            for(int i=thickness; i<w-thickness; i++) {
+                QRgb col= img.pixel(i,j);
+                if (col == bgColor) continue;
+                float g = qGray(col);
+                if (inverse)
+                    g = (255-g)/(255-gray);
+                else
+                    g = g/gray;
+                for(int n=-thickness; n<=thickness; n++) {
+                    for(int m=-thickness; m<=thickness; m++) {
+                        int d = n*n+m*m;
+                        if (d>md) continue;
+                        float f = bmap[d];
+                        amap[(i+m)+(j+n)*w] += (g*f);
+                    }
+                }
+            }
+        }
+
+        QImage res(w,h,32);
+        res.setAlphaBuffer(true);
+        int r = qRed(color);
+        int g = qGreen(color);
+        int b = qBlue(color);
+
+        for(int j=0; j<h; j++) {
+            for(int i=0; i<w; i++) {
+                res.setPixel(i,j, qRgba(r,g,b,(int)(amap[i+j*w]*factor*255.0)));
+            }
+        }
+
+        pt->drawImage(x-thickness, y-thickness, res, 0, 0, -1, -1, Qt::DiffuseAlphaDither | Qt::ColorOnly | Qt::PreferDither);
+    }
+    // Paint next shadow effect
+    if (shadow->next) paintShadow(pt, f, _tx, _ty, shadow->next);
+}
 
 /**
  * Distributes pixels to justify text.
@@ -268,7 +280,7 @@ void InlineTextBox::paintDecoration( QPainter *pt, const Font *f, int _tx, int _
  *	amount of pixels distributed during this call subtracted.
  * @return number of pixels to distribute
  */
-inline int justifyWidth(int &numSpaces, int &toAdd) {
+static inline int justifyWidth(int &numSpaces, int &toAdd) {
   int a = 0;
   if ( numSpaces ) {
     a = toAdd/numSpaces;
@@ -518,8 +530,10 @@ RenderText::RenderText(DOM::NodeImpl* node, DOMStringImpl *_str)
 void RenderText::setStyle(RenderStyle *_style)
 {
     if ( style() != _style ) {
-        bool changedText = (!style() && (_style->textTransform() != TTNONE)) ||
-            ((style() && style()->textTransform() != _style->textTransform()) );
+        bool changedText = ((!style() && ( _style->textTransform() != TTNONE ||
+                                           !_style->preserveLF() || !_style->preserveWS() )) ||
+                            (style() && (style()->textTransform() != _style->textTransform() ||
+                                         style()->whiteSpace() != _style->whiteSpace())));
 
         RenderObject::setStyle( _style );
         m_lineHeight = RenderObject::lineHeight(false);
@@ -577,7 +591,7 @@ InlineTextBox * RenderText::findInlineTextBox( int offset, int &pos, bool checkF
     // and return pos, which is the position of the char in the run.
 
     // FIXME: make this use binary search? Dirk says it won't work :-( (LS)
-
+    (void)checkFirstLetter;
 #if 0
     if (checkFirstLetter && forcedMinOffset()) {
 //        kdDebug(6040) << "checkFirstLetter: forcedMinOffset: " << forcedMinOffset() << endl;
@@ -627,7 +641,7 @@ InlineTextBox * RenderText::findInlineTextBox( int offset, int &pos, bool checkF
     return s;
 }
 
-bool RenderText::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty, HitTestAction /*hitTestAction*/, bool inBox)
+bool RenderText::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty, HitTestAction /*hitTestAction*/, bool /*inBox*/)
 {
     assert(parent());
 
@@ -754,14 +768,12 @@ void RenderText::caretPos(int offset, int flags, int &_x, int &_y, int &width, i
 #if 0
   kdDebug(6040) << "_x="<<_x << " s->m_x="<<s->m_x
   		<< " s->m_start"<<s->m_start
-		<< " s->m_len"<<s->m_len << endl;
+		<< " s->m_len" << s->m_len << " _y=" << _y << endl;
 #endif
 
   int absx, absy;
 
-  RenderObject *cb = containingBlock();
-
-  if (cb && cb != this && cb->absolutePosition(absx,absy))
+  if (absolutePosition(absx,absy))
   {
     //kdDebug(6040) << "absx=" << absx << " absy=" << absy << endl;
     _x += absx;
@@ -848,8 +860,8 @@ void RenderText::paint( PaintInfo& pI, int tx, int ty)
     InlineTextBox f(0, pI.r.top()-ty);
     int si = m_lines.findFirstMatching(&f);
     // something matching found, find the first one to paint
-    bool isPrinting = (pI.p->device()->devType() == QInternal::Printer);
-    if (isPrinting && pI.phase == PaintActionSelection) return;
+    bool isStatic = canvas()->staticMode();
+    if (isStatic && pI.phase == PaintActionSelection) return;
 
     if(si >= 0)
     {
@@ -860,7 +872,7 @@ void RenderText::paint( PaintInfo& pI, int tx, int ty)
         // Now calculate startPos and endPos, for painting selection.
         // We paint selection while endPos > 0
         int endPos, startPos;
-        if (!isPrinting && (selectionState() != SelectionNone)) {
+        if (!isStatic && (selectionState() != SelectionNone)) {
             if (selectionState() == SelectionInside) {
                 //kdDebug(6040) << this << " SelectionInside -> 0 to end" << endl;
                 startPos = 0;
@@ -878,7 +890,7 @@ void RenderText::paint( PaintInfo& pI, int tx, int ty)
         InlineTextBox* s;
 	const Font *font = &style()->htmlFont();
 
-        bool haveSelection = !isPrinting && selectionState() != SelectionNone && startPos != endPos;
+        bool haveSelection = !isStatic && selectionState() != SelectionNone && startPos != endPos;
         if (!haveSelection && pI.phase == PaintActionSelection)
             // When only painting the selection, don't bother to paint if there
             return;
@@ -888,25 +900,6 @@ void RenderText::paint( PaintInfo& pI, int tx, int ty)
         do {
             s = m_lines[si];
 
-	    if (isPrinting)
-	    {
-                int lh = lineHeight( false ) + paddingBottom() + borderBottom();
-                if (ty+s->m_y < pI.r.y())
-                {
-                   // This has been painted already we suppose.
-                   continue;
-                }
-
-                if (ty+lh+s->m_y > pI.r.bottom())
-                {
-                   RenderCanvas *rootObj = canvas();
-                   if (ty+s->m_y < rootObj->truncatedAt())
-                      rootObj->setBestTruncatedAt(ty+s->m_y, this);
-                   // Let's stop here.
-                   break;
-                }
-            }
-
             RenderStyle* _style = pseudoStyle && s->m_firstLine ? pseudoStyle : style();
             int d = _style->textDecorationsInEffect();
 
@@ -919,9 +912,24 @@ void RenderText::paint( PaintInfo& pI, int tx, int ty)
             if(_style->color() != pI.p->pen().color())
                 pI.p->setPen(_style->color());
 
-	    if (s->m_len > 0 && pI.phase != PaintActionSelection) {
+#ifdef APPLE_CHANGES
+            // Set a text shadow if we have one.
+            bool setShadow = false;
+            if (_style->textShadow()) {
+                p->setShadow(_style->textShadow()->x, _style->textShadow()->y,
+                             _style->textShadow()->blur, _style->textShadow()->color);
+                setShadow = true;
+            }
+#endif
+
+            if (s->m_len > 0 && pI.phase != PaintActionSelection) {
 	        if (!haveSelection) {
 	            //kdDebug( 6040 ) << "RenderObject::paintObject(" << QConstString(str->s + s->m_start, s->m_len).string() << ") at(" << s->m_x+tx << "/" << s->m_y+ty << ")" << endl;
+#ifndef APPLE_CHANGES
+                    if (_style->textShadow())
+                        s->paintShadow(pI.p, font, tx, ty, _style->textShadow());
+#endif
+// kdDebug(6040) << QConstString(str->s + s->m_start, s->m_len).string().left(40) << endl;
 		    font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline, str->s, str->l, s->m_start, s->m_len,
 				   s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR);
 	        }
@@ -932,6 +940,10 @@ void RenderText::paint( PaintInfo& pI, int tx, int ty)
 // selected text is always separate in konqueror
 #ifdef APPLE_CHANGES
                     if (paintSelectedTextSeparately) {
+#endif
+#ifndef APPLE_CHANGES
+                        if (_style->textShadow())
+                            s->paintShadow(pI.p, font, tx, ty, _style->textShadow());
 #endif
                         if (sPos >= ePos)
                             font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline,
@@ -939,13 +951,13 @@ void RenderText::paint( PaintInfo& pI, int tx, int ty)
                                            s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR);
                         else {
                             if (sPos-1 >= 0)
-                                font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline, str->s,
-                                            str->l, s->m_start, s->m_len,
-                                            s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR, 0, sPos);
+                                font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline,
+                                               str->s, str->l, s->m_start, s->m_len,
+                                               s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR, 0, sPos);
                             if (ePos < s->m_len)
-                                font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline, str->s,
-                                            str->l, s->m_start, s->m_len,
-                                            s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR, ePos, s->m_len);
+                                font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline,
+                                               str->s, str->l, s->m_start, s->m_len,
+                                               s->m_toAdd, s->m_reversed ? QPainter::RTL : QPainter::LTR, ePos, s->m_len);
                         }
 #ifdef APPLE_CHANGES
                     }
@@ -981,9 +993,9 @@ void RenderText::paint( PaintInfo& pI, int tx, int ty)
             {
                 int h = lineHeight( false ) + paddingTop() + paddingBottom() + borderTop() + borderBottom();
                 QColor c2 = QColor("#0000ff");
-                drawBorder(p, tx + s->m_x, ty + s->m_y, tx + s->m_x + 1, ty + s->m_y + h,
+                drawBorder(pI.p, tx + s->m_x, ty + s->m_y, tx + s->m_x + 1, ty + s->m_y + h,
                               RenderObject::BSLeft, c2, c2, SOLID, 1, 1);
-                drawBorder(p, tx + s->m_x + s->m_width, ty + s->m_y, tx + s->m_x + s->m_width + 1, ty + s->m_y + h,
+                drawBorder(pI.p, tx + s->m_x + s->m_width, ty + s->m_y, tx + s->m_x + s->m_width + 1, ty + s->m_y + h,
                               RenderObject::BSRight, c2, c2, SOLID, 1, 1);
             }
 #endif
@@ -1012,25 +1024,19 @@ void RenderText::calcMinMaxWidth()
     const Font *f = htmlFont( false );
     int wordSpacing = style()->wordSpacing();
     int len = str->l;
-    bool ignoringSpaces = false;
     bool isSpace = false;
-    bool isPre = style()->whiteSpace() == PRE;
     bool firstWord = true;
     bool firstLine = true;
     for(int i = 0; i < len; i++)
     {
         unsigned short c = str->s[i].unicode();
-        bool previousCharacterIsSpace = isSpace;
         bool isNewline = false;
 
+        // If line-breaks survive to here they are preserved
         if ( c == '\n' ) {
-            if ( isPre ) {
-                m_hasBreak = true;
-                isNewline = true;
-                isSpace = false;
-            }
-            else
-                isSpace = true;
+            m_hasBreak = true;
+            isNewline = true;
+            isSpace = false;
         } else
             isSpace = c == ' ';
 
@@ -1038,18 +1044,13 @@ void RenderText::calcMinMaxWidth()
             m_hasBeginWS = true;
         if ((isSpace || isNewline) && i == len-1)
             m_hasEndWS = true;
-
-        if (!ignoringSpaces && !isPre && previousCharacterIsSpace && isSpace)
-            ignoringSpaces = true;
-
-        if (ignoringSpaces && !isSpace)
-            ignoringSpaces = false;
-
-        if (ignoringSpaces)
+        
+        if (i && c == SOFT_HYPHEN)
             continue;
 
         int wordlen = 0;
-        while( i+wordlen < len && !(isBreakable( str->s, i+wordlen, str->l )) )
+        while( i+wordlen < len && (i+wordlen == 0 || str->s[i+wordlen].unicode() != SOFT_HYPHEN) &&
+               !(isBreakable( str->s, i+wordlen, str->l )) )
             wordlen++;
 
         if (wordlen)
@@ -1079,14 +1080,14 @@ void RenderText::calcMinMaxWidth()
         }
         else {
             // Nowrap can never be broken, so don't bother setting the
-            // breakable character boolean.
-            if (style()->whiteSpace() == NORMAL || isNewline)
+            // breakable character boolean. Pre can only be broken if we encounter a newline.
+            if (style()->autoWrap() || isNewline)
                 m_hasBreakableChar = true;
 
             if(currMinWidth > m_minWidth) m_minWidth = currMinWidth;
             currMinWidth = 0;
 
-            if (isNewline)
+            if (isNewline) // Only set if isPre was true and we saw a newline.
             {
                 if ( firstLine ) {
                     firstLine = false;
@@ -1106,13 +1107,13 @@ void RenderText::calcMinMaxWidth()
     if(currMinWidth > m_minWidth) m_minWidth = currMinWidth;
     if(currMaxWidth > m_maxWidth) m_maxWidth = currMaxWidth;
 
-    if (style()->whiteSpace() != NORMAL)
+    if (!style()->autoWrap()) {
         m_minWidth = m_maxWidth;
-
-    if (isPre) {
-        if (firstLine)
-            m_beginMinWidth = m_maxWidth;
-        m_endMinWidth = currMaxWidth;
+        if (style()->preserveLF()) {
+            if (firstLine)
+                m_beginMinWidth = m_maxWidth;
+            m_endMinWidth = currMaxWidth;
+        }
     }
 
     setMinMaxKnown();
@@ -1150,18 +1151,26 @@ const QFont &RenderText::font()
 void RenderText::setText(DOMStringImpl *text, bool force)
 {
     if( !force && str == text ) return;
-    if(str) str->deref();
-    str = text;
+
+    DOMStringImpl *oldstr = str;
+    if(text && style())
+        str = text->collapseWhiteSpace(style()->preserveLF(), style()->preserveWS());
+    else
+        str = text;
+    if(str) str->ref();
+    if(oldstr) oldstr->deref();
 
     if ( str && style() ) {
-	switch(style()->textTransform()) {
+        oldstr = str;
+        switch(style()->textTransform()) {
 	case CAPITALIZE:   str = str->capitalize();  break;
 	case UPPERCASE:   str = str->upper();       break;
 	case LOWERCASE:  str = str->lower();       break;
 	case NONE:
 	default:;
 	}
-	str->ref();
+        str->ref();
+        oldstr->deref();
     }
 
     // ### what should happen if we change the text of a
@@ -1336,16 +1345,23 @@ void RenderText::trimmedMinMaxWidth(short& beginMinW, bool& beginWS,
                                     short& beginMaxW, short& endMaxW,
                                     short& minW, short& maxW, bool& stripFrontSpaces)
 {
-    int len = str->l;
-    bool isPre = style()->whiteSpace() == PRE;
-    if (isPre)
+    bool preserveWS = style()->preserveWS();
+    bool preserveLF = style()->preserveLF();
+    bool autoWrap = style()->autoWrap();
+    if (preserveWS)
         stripFrontSpaces = false;
 
+    int len = str->l;
+    if (len == 0 || (stripFrontSpaces && str->containsOnlyWhitespace())) {
+        maxW = 0;
+        hasBreak = false;
+        return;
+    }
+
     minW = m_minWidth;
     maxW = m_maxWidth;
     beginWS = stripFrontSpaces ? false : m_hasBeginWS;
-    // Handle the case where all space got stripped.
-    endWS = stripFrontSpaces && len > 0 && str->containsOnlyWhitespace() ? false : m_hasEndWS;
+    endWS = m_hasEndWS;
 
     beginMinW = m_beginMinWidth;
     endMinW = m_endMinWidth;
@@ -1353,19 +1369,16 @@ void RenderText::trimmedMinMaxWidth(short& beginMinW, bool& beginWS,
     hasBreakableChar = m_hasBreakableChar;
     hasBreak = m_hasBreak;
 
-    if (len == 0)
-        return;
-
-    if (stripFrontSpaces && str->s[0].direction() == QChar::DirWS) {
+    if (stripFrontSpaces && (str->s[0].direction() == QChar::DirWS || (!preserveLF && str->s[0] == '\n'))) {
         const Font *f = htmlFont( false );
         QChar space[1]; space[0] = ' ';
         int spaceWidth = f->width(space, 1, 0);
         maxW -= spaceWidth;
     }
 
-    stripFrontSpaces = !isPre && endWS;
+    stripFrontSpaces = !preserveWS && m_hasEndWS;
 
-    if (style()->whiteSpace() == NOWRAP)
+    if (!autoWrap)
         minW = maxW;
     else if (minW > maxW)
         minW = maxW;
@@ -1393,13 +1406,15 @@ void RenderText::trimmedMinMaxWidth(short& beginMinW, bool& beginWS,
                     beginMaxW = endMaxW;
                 }
                 i += linelen;
-                if (i == len-1)
-                    endMaxW = 0;
             }
             else if (firstLine) {
                 beginMaxW = 0;
                 firstLine = false;
             }
+	    if (i == len-1)
+	        // A <pre> run that ends with a newline, as in, e.g.,
+	        // <pre>Some text\n\n<span>More text</pre>
+	        endMaxW = 0;
         }
     }
 }
