@@ -4,6 +4,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
+ *           (C) 2003 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -17,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 #ifndef _DOM_NodeImpl_h_
@@ -35,6 +36,7 @@
 
 class QPainter;
 template <class type> class QPtrList;
+template <class type> class QValueList;
 class KHTMLView;
 class QRect;
 class QMouseEvent;
@@ -44,6 +46,7 @@ class QTextStream;
 namespace khtml {
     class RenderStyle;
     class RenderObject;
+    class RenderArena;
 }
 
 namespace DOM {
@@ -67,10 +70,45 @@ private:
     DocumentImpl *doc;
 };
 
+struct RegisteredListenerList {
+    RegisteredListenerList() : listeners(0)
+    {}
+
+    ~RegisteredListenerList();
+
+    void addEventListener(int id, EventListener *listener, const bool useCapture);
+    void removeEventListener(int id, EventListener *listener, bool useCapture);
+
+    void setHTMLEventListener(int id, EventListener *listener);
+    EventListener *getHTMLEventListener(int id);
+
+    bool hasEventListener(int id);
+    void clear();
+
+    //### KDE4: should disappear
+    bool stillContainsListener(const RegisteredEventListener& listener);
+
+    QValueList<RegisteredEventListener>* listeners;//The actual listener list - may be 0
+private:
+    bool isHTMLEventListener(EventListener* listener);
+};
+
+
 // this class implements nodes, which can have a parent but no children:
 #define NodeImpl_IdNSMask    0xffff0000
 #define NodeImpl_IdLocalMask 0x0000ffff
 
+const Q_UINT16 noNamespace = 0;
+const Q_UINT16 anyNamespace = 0xffff;
+const Q_UINT16 xhtmlNamespace = 1;
+const Q_UINT16 anyLocalName = 0xffff;
+
+inline Q_UINT16 localNamePart(Q_UINT32 id) { return id & NodeImpl_IdLocalMask; }
+inline Q_UINT16 namespacePart(Q_UINT32 id) { return (((unsigned int)id) & NodeImpl_IdNSMask) >> 16; }
+inline Q_UINT32 makeId(Q_UINT16 n, Q_UINT16 l) { return (n << 16) | l; }
+
+const Q_UINT32 anyQName = makeId(anyNamespace, anyLocalName);
+
 class NodeImpl : public khtml::TreeShared<NodeImpl>
 {
     friend class DocumentImpl;
@@ -89,6 +127,8 @@ public:
     virtual NodeListImpl *childNodes();
     virtual NodeImpl *firstChild() const;
     virtual NodeImpl *lastChild() const;
+    // insertBefore, replaceChild and appendChild also close newChild
+    // unlike the speed optimized addChild (which is used by the parser)
     virtual NodeImpl *insertBefore ( NodeImpl *newChild, NodeImpl *refChild, int &exceptioncode );
     virtual NodeImpl *replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
     virtual NodeImpl *removeChild ( NodeImpl *oldChild, int &exceptioncode );
@@ -179,19 +219,17 @@ public:
     virtual void sheetLoaded() {}
 
     bool hasID() const      { return m_hasId; }
-    bool hasStyle() const   { return m_hasStyle; }
     bool active() const     { return m_active; }
     bool focused() const { return m_focused; }
     bool attached() const   { return m_attached; }
+    bool closed() const     { return m_closed; }
     bool changed() const    { return m_changed; }
     bool hasChangedChild() const { return m_hasChangedChild; }
     bool hasAnchor() const { return m_hasAnchor; }
     bool inDocument() const { return m_inDocument; }
-    bool styleElement() const { return m_styleElement; }
     bool implicitNode() const { return m_implicit; }
     bool htmlCompat() const { return m_htmlCompat; }
     void setHasID(bool b=true) { m_hasId = b; }
-    void setHasStyle(bool b=true) { m_hasStyle = b; }
     void setHasChangedChild( bool b = true ) { m_hasChangedChild = b; }
     void setInDocument(bool b=true) { m_inDocument = b; }
     void setHTMLCompat(bool b) { m_htmlCompat = b; }
@@ -222,7 +260,6 @@ public:
 
     void addEventListener(int id, EventListener *listener, const bool useCapture);
     void removeEventListener(int id, EventListener *listener, bool useCapture);
-    void removeHTMLEventListener(int id);
     void setHTMLEventListener(int id, EventListener *listener);
     EventListener *getHTMLEventListener(int id);
 
@@ -274,6 +311,7 @@ public:
 
     khtml::RenderObject *renderer() const { return m_render; }
     khtml::RenderObject *nextRenderer();
+    khtml::RenderObject *previousRenderer();
     void setRenderer(khtml::RenderObject* renderer) { m_render = renderer; }
 
     void checkSetPrefix(const DOMString &_prefix, int &exceptioncode);
@@ -312,8 +350,18 @@ public:
      */
     virtual void detach();
 
+    /**
+     * Notifies the node that no more children will be added.
+     */
+    virtual void close();
+
     void closeRenderer();
 
+    void createRendererIfNeeded();
+    virtual khtml::RenderStyle *styleForRenderer(khtml::RenderObject *parent);
+    virtual bool rendererIsNeeded(khtml::RenderStyle *);
+    virtual khtml::RenderObject *createRenderer(khtml::RenderArena *, khtml::RenderStyle *);
+
     // -----------------------------------------------------------------------------
     // Methods for maintaining the state of the element between history navigation
 
@@ -371,6 +419,17 @@ public:
     virtual void childrenChanged();
 
     virtual DOMString toString() const = 0;
+    /**
+     * Sometimes we need to get the string between two points on the DOM graph.  Use this function to do this.
+     * For example, when the user copies some selected text to the clipboard as html.
+     * @param selectionStart Where to start the selection.  If selectionStart != this, it is assumed we are after the start point
+     * @param selectionEnd   Where to end the selection.  If selectionEnd != this, it is assumed we are before the end point (unless found is true)
+     * @param startOffset    Number of characters into the text in selectionStart that the start of the selection is.
+     * @param endOffset      Number of characters into the text in selectionEnd that the end of the selection is.
+     * @param found          When this is set to true, don't print anymore but closing tags.
+     * @return An html formatted string for this node and its children between the selectionStart and selectionEnd.
+     */
+    virtual DOMString selectionToString(NodeImpl * /*selectionStart*/, NodeImpl * /*selectionEnd*/, int /*startOffset*/, int /*endOffset*/, bool &/*found*/) const { return toString(); }
 
 private: // members
     DocumentPtr *document;
@@ -378,27 +437,28 @@ private: // members
     NodeImpl *m_next;
 protected:
     khtml::RenderObject *m_render;
-    QPtrList<RegisteredEventListener> *m_regdListeners;
+    RegisteredListenerList m_regdListeners;
 
     unsigned short m_tabIndex : 15;
     bool m_hasTabIndex  : 1;
 
     bool m_hasId : 1;
-    bool m_hasStyle : 1;
     bool m_attached : 1;
+    bool m_closed : 1;
     bool m_changed : 1;
     bool m_hasChangedChild : 1;
     bool m_inDocument : 1;
     bool m_hasAnchor : 1;
-
     bool m_specified : 1; // used in AttrImpl. Accessor functions there
+    
     bool m_focused : 1;
     bool m_active : 1;
-    bool m_styleElement : 1; // contains stylesheet text
     bool m_implicit : 1; // implicitely generated by the parser
     bool m_rendererNeedsClose : 1;
     bool m_htmlCompat : 1; // true if element was created in HTML compat mode
     bool m_unused : 1;
+    bool m_unused2 : 1;
+    bool m_unused3 : 1;    
 };
 
 // this is the full Node Implementation with parents and children.
@@ -462,7 +522,63 @@ class NodeImpl;
 class NodeListImpl : public khtml::Shared<NodeListImpl>
 {
 public:
-    virtual ~NodeListImpl() {}
+    //Type of the item stored in the cache.
+    enum Type {
+        UNCACHEABLE, //Too complex to be cached like this
+        CHILD_NODES,
+        LAST_NODE_LIST = CHILD_NODES
+    };
+
+    struct CacheKey
+    {
+        NodeImpl* baseNode;
+        int       type;
+
+        CacheKey(): type(UNCACHEABLE) {}
+
+        CacheKey(NodeImpl* _baseNode, int _type):
+            baseNode(_baseNode), type(_type)
+        {}
+
+        int hash() const
+        {
+            return int(reinterpret_cast<unsigned long>(baseNode) >> 2) ^
+                       (unsigned(type) << 26);
+        }
+
+        bool operator==(const CacheKey& other) const
+        {
+            return baseNode == other.baseNode &&
+                   type     == other.type;
+        }
+    };
+
+    struct Cache: public khtml::Shared<Cache>
+    {
+        static Cache* make() { return new Cache; }
+    
+        CacheKey key;//### We must store this in here due to QCache in Qt3 sucking
+
+        unsigned int version;
+        union
+        {
+            NodeImpl*    node;
+            unsigned int index;
+        } current;
+        unsigned int position;
+        unsigned int length;
+        bool         hasLength;
+        
+        void updateNodeListInfo(DocumentImpl* doc);
+
+        virtual void clear(DocumentImpl* doc);
+        virtual ~Cache();
+    };
+
+    typedef Cache* CacheFactory();
+
+    NodeListImpl(NodeImpl* node, int type, CacheFactory* factory = 0);
+    virtual ~NodeListImpl();
 
     // DOM methods & attributes for NodeList
     virtual unsigned long length() const;
@@ -471,27 +587,28 @@ public:
     // Other methods (not part of DOM)
 
 protected:
+    virtual unsigned long calcLength(NodeImpl *start) const;
     // helper functions for searching all ElementImpls in a tree
-    unsigned long recursiveLength(NodeImpl *start) const;
-    NodeImpl *recursiveItem ( NodeImpl *start, unsigned long &offset ) const;
-    virtual bool elementMatches( NodeImpl *testNode ) const = 0;
+
+    NodeImpl *recursiveItem    ( NodeImpl* absStart, NodeImpl *start, unsigned long &offset ) const;
+    NodeImpl *recursiveItemBack( NodeImpl* absStart, NodeImpl *start, unsigned long &offset ) const;
+
+    // Override this to determine what nodes to return. Set doRecurse to 
+    // false if the children of this node do not need to be entered.
+    virtual bool nodeMatches( NodeImpl *testNode, bool& doRecurse ) const = 0;
+
+    NodeImpl*      m_refNode;
+    mutable Cache* m_cache;
 };
 
 class ChildNodeListImpl : public NodeListImpl
 {
 public:
+ 
     ChildNodeListImpl( NodeImpl *n);
-    virtual ~ChildNodeListImpl();
-
-    // DOM methods overridden from  parent classes
-
-    virtual unsigned long length() const;
-    virtual NodeImpl *item ( unsigned long index ) const;
-
+    
 protected:
-    virtual bool elementMatches( NodeImpl *testNode ) const;
-
-    NodeImpl *refNode;
+    virtual bool nodeMatches( NodeImpl *testNode, bool& doRecurse ) const;
 };
 
 
@@ -503,19 +620,11 @@ class TagNodeListImpl : public NodeListImpl
 public:
     TagNodeListImpl( NodeImpl *n, NodeImpl::Id id );
     TagNodeListImpl( NodeImpl *n, const DOMString &namespaceURI, const DOMString &localName );
-    virtual ~TagNodeListImpl();
-
-    // DOM methods overridden from  parent classes
-
-    virtual unsigned long length() const;
-    virtual NodeImpl *item ( unsigned long index ) const;
 
     // Other methods (not part of DOM)
 
 protected:
-    virtual bool elementMatches( NodeImpl *testNode ) const;
-
-    NodeImpl *m_refNode;
+    virtual bool nodeMatches( NodeImpl *testNode, bool& doRecurse ) const;
     NodeImpl::Id m_id;
     DOMString m_namespaceURI;
     DOMString m_localName;
@@ -533,32 +642,12 @@ class NameNodeListImpl : public NodeListImpl
 {
 public:
     NameNodeListImpl( NodeImpl *doc, const DOMString &t );
-    virtual ~NameNodeListImpl();
-
-    // DOM methods overridden from  parent classes
-
-    virtual unsigned long length() const;
-    virtual NodeImpl *item ( unsigned long index ) const;
 
     // Other methods (not part of DOM)
 
 protected:
-    virtual bool elementMatches( NodeImpl *testNode ) const;
+    virtual bool nodeMatches( NodeImpl *testNode, bool& doRecurse ) const;
 
-    NodeImpl *refNode;
-    DOMString nodeName;
-};
-
-/**
- * NodeList which lists all Nodes in a document with a given tag name
- * _and_ a given value for the name attribute (combination of TagNodeListImpl and NameNodeListImpl)
- */
-class NamedTagNodeListImpl : public TagNodeListImpl
-{
-public:
-    NamedTagNodeListImpl( NodeImpl *n, NodeImpl::Id tagId, const DOMString& name );
-protected:
-    virtual bool elementMatches( NodeImpl *testNode ) const;
     DOMString nodeName;
 };
 
