@@ -18,10 +18,12 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
+#include "kfiledialog.h"
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -38,6 +40,7 @@
 #include <qtooltip.h>
 #include <qtimer.h>
 #include <qwhatsthis.h>
+#include <qfiledialog.h>
 
 #include <kaccel.h>
 #include <kaction.h>
@@ -53,8 +56,8 @@
 #include <kimageio.h>
 #include <kio/job.h>
 #include <kio/netaccess.h>
-#include <kio/previewjob.h>
 #include <kio/scheduler.h>
+#include <kio/kservicetypefactory.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kmimetype.h>
@@ -79,7 +82,6 @@
 #include <kdirselectdialog.h>
 #include <kfileview.h>
 #include <krecentdocument.h>
-#include <kfiledialog.h>
 #include <kfilefiltercombo.h>
 #include <kdiroperator.h>
 #include <kimagefilepreview.h>
@@ -87,6 +89,11 @@
 #include <kfilespeedbar.h>
 #include <kfilebookmarkhandler.h>
 
+#ifdef Q_WS_X11
+#include <X11/Xlib.h>
+#include <fixx11h.h>
+#endif
+
 enum Buttons { HOTLIST_BUTTON,
                PATH_COMBO, CONFIGURE_BUTTON };
 
@@ -143,9 +150,6 @@ struct KFileDialogPrivate
     // setting it again and again, we have this nice little boolean :)
     bool hasView :1;
 
-    // do we show the speedbar for the first time?
-    bool initializeSpeedbar :1;
-
     bool hasDefaultFilter :1; // necessary for the operationMode
     KFileDialog::OperationMode operationMode;
 
@@ -310,7 +314,7 @@ KURL KFileDialog::getCompleteURL(const QString &_url)
 
     if ( KURL::isRelativeURL(url) ) // only a full URL isn't relative. Even /path is.
     {
-        if (!url.isEmpty() && url[0] == '/' ) // absolute path
+        if (!url.isEmpty() && !QDir::isRelativePath(url) ) // absolute path
             u.setPath( url );
         else
         {
@@ -473,8 +477,8 @@ void KFileDialog::slotOk()
                 }
                 else { // d->url is not a directory,
                     // maybe we are in File(s) | Directory mode
-                    if ( mode() & KFile::File == KFile::File ||
-                        mode() & KFile::Files == KFile::Files )
+                    if ( (mode() & KFile::File) == KFile::File ||
+                        (mode() & KFile::Files) == KFile::Files )
                         done = false;
                 }
             }
@@ -766,31 +770,35 @@ void KFileDialog::setLocationText( const QString& text )
     connect( locationEdit, SIGNAL( textChanged( const QString& ) ),
              SLOT( slotLocationChanged( const QString& )) );
     locationEdit->setEditText( text );
+
+    // don't change selection when user has clicked on an item
+    if ( d->operationMode == Saving && !locationEdit->isVisible())
+       setNonExtSelection();
 }
 
-static QString autocompletionWhatsThisText = i18n("<p>While typing in the text area, you may be presented "
+static const char autocompletionWhatsThisText[] = I18N_NOOP("<p>While typing in the text area, you may be presented "
                                                   "with possible matches. "
                                                   "This feature can be controlled by clicking with the right mouse button "
-                                                  "and selecting a preferred mode from the <b>Text Completion</b> menu.") + "</qt>";
+                                                  "and selecting a preferred mode from the <b>Text Completion</b> menu.")  "</qt>";
 void KFileDialog::updateLocationWhatsThis (void)
 {
     QString whatsThisText;
     if (d->operationMode == KFileDialog::Saving)
     {
         whatsThisText = "<qt>" + i18n("This is the name to save the file as.") +
-                             autocompletionWhatsThisText;
+                             i18n (autocompletionWhatsThisText);
     }
     else if (ops->mode() & KFile::Files)
     {
         whatsThisText = "<qt>" + i18n("This is the list of files to open. More than "
                              "one file can be specified by listing several "
                              "files, separated by spaces.") +
-                              autocompletionWhatsThisText;
+                              i18n (autocompletionWhatsThisText);
     }
     else
     {
         whatsThisText = "<qt>" + i18n("This is the name of the file to open.") +
-                             autocompletionWhatsThisText;
+                             i18n (autocompletionWhatsThisText);
     }
 
     QWhatsThis::add(d->locationLabel, whatsThisText);
@@ -805,6 +813,7 @@ void KFileDialog::init(const QString& startDir, const QString& filter, QWidget*
     d->boxLayout = 0;
     d->keepLocation = false;
     d->operationMode = Opening;
+    d->bookmarkHandler = 0;
     d->hasDefaultFilter = false;
     d->hasView = false;
     d->mainWidget = new QWidget( this, "KFileDialog::mainWidget");
@@ -818,10 +827,6 @@ void KFileDialog::init(const QString& startDir, const QString& filter, QWidget*
     d->autoSelectExtCheckBox = 0; // delayed loading
     d->autoSelectExtChecked = false;
     d->urlBar = 0; // delayed loading
-    KConfig *config = KGlobal::config();
-    KConfigGroupSaver cs( config, ConfigGroup );
-    d->initializeSpeedbar = config->readBoolEntry( "Set speedbar defaults",
-                                                   true );
 
     QtMsgHandler oldHandler = qInstallMsgHandler( silenceQToolBar );
     toolbar = new KToolBar( d->mainWidget, "KFileDialog::toolbar", true);
@@ -830,10 +835,11 @@ void KFileDialog::init(const QString& startDir, const QString& filter, QWidget*
 
     d->pathCombo = new KURLComboBox( KURLComboBox::Directories, true,
                                      toolbar, "path combo" );
-    QToolTip::add( d->pathCombo, i18n("Often used folders") );
-    QWhatsThis::add( d->pathCombo, "<qt>" + i18n("Commonly used locations are listed here. "
+    QToolTip::add( d->pathCombo, i18n("Current location") );
+    QWhatsThis::add( d->pathCombo, "<qt>" + i18n("This is the currently listed location. "
+                                                 "The drop-down list also lists commonly used locations. "
                                                  "This includes standard locations, such as your home folder, as well as "
-                                                 "locations that have been visited recently.") + autocompletionWhatsThisText);
+                                                 "locations that have been visited recently.") + i18n (autocompletionWhatsThisText));
 
     KURL u;
     u.setPath( QDir::rootDirPath() );
@@ -849,7 +855,7 @@ void KFileDialog::init(const QString& startDir, const QString& filter, QWidget*
 
     KURL docPath;
     docPath.setPath( KGlobalSettings::documentPath() );
-    if ( (u.path(+1) != docPath.path(+1)) && 
+    if ( (u.path(+1) != docPath.path(+1)) &&
          QDir(docPath.path(+1)).exists() )
     {
         text = i18n("Documents: %1").arg( docPath.path( +1 ) );
@@ -864,8 +870,6 @@ void KFileDialog::init(const QString& startDir, const QString& filter, QWidget*
                                  KMimeType::pixmapForURL( u, 0, KIcon::Small ),
                                  text );
 
-    u.setPath( "/tmp" );
-
     d->url = getStartURL( startDir, d->fileClass );
     d->selection = d->url.url();
 
@@ -915,27 +919,18 @@ void KFileDialog::init(const QString& startDir, const QString& filter, QWidget*
     coll->action( "mkdir" )->plug( toolbar );
     coll->action( "mkdir" )->setWhatsThis(i18n("Click this button to create a new folder."));
 
-    d->bookmarkHandler = new KFileBookmarkHandler( this );
-    toolbar->insertButton(QString::fromLatin1("bookmark"),
-                          (int)HOTLIST_BUTTON, true,
-                          i18n("Bookmarks"));
-    toolbar->getButton(HOTLIST_BUTTON)->setPopup( d->bookmarkHandler->menu(),
-                                                  true);
-    QWhatsThis::add(toolbar->getButton(HOTLIST_BUTTON),
-                    i18n("<qt>This button allows you to bookmark specific locations. "
-                         "Click on this button to open the bookmark menu where you may add, "
-                         "edit or select a bookmark.<p>"
-                         "These bookmarks are specific to the file dialog, but otherwise operate "
-                         "like bookmarks elsewhere in KDE.</qt>"));
-    connect( d->bookmarkHandler, SIGNAL( openURL( const QString& )),
-             SLOT( enterURL( const QString& )));
-
     KToggleAction *showSidebarAction =
         new KToggleAction(i18n("Show Quick Access Navigation Panel"), Key_F9, coll,"toggleSpeedbar");
     showSidebarAction->setCheckedState(i18n("Hide Quick Access Navigation Panel"));
     connect( showSidebarAction, SIGNAL( toggled( bool ) ),
              SLOT( toggleSpeedbar( bool )) );
 
+    KToggleAction *showBookmarksAction =
+            new KToggleAction(i18n("Show Bookmarks"), 0, coll, "toggleBookmarks");
+    showBookmarksAction->setCheckedState(i18n("Hide Bookmarks"));
+    connect( showBookmarksAction, SIGNAL( toggled( bool ) ),
+             SLOT( toggleBookmarks( bool )) );
+
     KActionMenu *menu = new KActionMenu( i18n("Configure"), "configure", this, "extra menu" );
     menu->setWhatsThis(i18n("<qt>This is the configuration menu for the file dialog. "
                             "Various options can be accessed from this menu including: <ul>"
@@ -955,6 +950,7 @@ void KFileDialog::init(const QString& startDir, const QString& filter, QWidget*
     coll->action( "show hidden" )->setShortcut(Key_F8);
     menu->insert( coll->action( "show hidden" ));
     menu->insert( showSidebarAction );
+    menu->insert( showBookmarksAction );
     coll->action( "preview" )->setShortcut(Key_F11);
     menu->insert( coll->action( "preview" ));
     coll->action( "separate dirs" )->setShortcut(Key_F12);
@@ -1037,6 +1033,7 @@ void KFileDialog::init(const QString& startDir, const QString& filter, QWidget*
 
     initGUI(); // activate GM
 
+    KConfig* config = KGlobal::config();
     readRecentFiles( config );
 
     adjustSize();
@@ -1297,6 +1294,8 @@ void KFileDialog::slotLocationChanged( const QString& text )
 {
     if ( text.isEmpty() && ops->view() )
         ops->view()->clearSelection();
+
+    updateFilter();
 }
 
 void KFileDialog::updateStatusLine(int /* dirs */, int /* files */)
@@ -1320,6 +1319,30 @@ QString KFileDialog::getOpenFileName(const QString& startDir,
     return dlg.selectedFile();
 }
 
+QString KFileDialog::getOpenFileNameWId(const QString& startDir,
+                                        const QString& filter,
+                                        WId parent_id, const QString& caption)
+{
+    QWidget* parent = QWidget::find( parent_id );
+    KFileDialog dlg(startDir, filter, parent, "filedialog", true);
+#ifdef Q_WS_X11
+    if( parent == NULL && parent_id != 0 )
+        XSetTransientForHint( qt_xdisplay(), dlg.winId(), parent_id );
+#else
+    // TODO
+#endif
+
+    dlg.setOperationMode( KFileDialog::Opening );
+
+    dlg.setMode( KFile::File | KFile::LocalOnly );
+    dlg.setCaption(caption.isNull() ? i18n("Open") : caption);
+
+    dlg.ops->clearHistory();
+    dlg.exec();
+
+    return dlg.selectedFile();
+}
+
 QStringList KFileDialog::getOpenFileNames(const QString& startDir,
                                           const QString& filter,
                                           QWidget *parent,
@@ -1377,12 +1400,17 @@ QString KFileDialog::getExistingDirectory(const QString& startDir,
                                           QWidget *parent,
                                           const QString& caption)
 {
+#ifdef Q_WS_WIN
+    return QFileDialog::getExistingDirectory(startDir, parent, "getExistingDirectory",
+                                             caption, true, true);
+#else
     KURL url = KDirSelectDialog::selectDirectory(startDir, true, parent,
                                                  caption);
     if ( url.isValid() )
         return url.path();
 
     return QString::null;
+#endif
 }
 
 KURL KFileDialog::getImageOpenURL( const QString& startDir, QWidget *parent,
@@ -1510,6 +1538,11 @@ QString KFileDialog::selectedFile() const
     {
        if (d->url.isLocalFile())
            return d->url.path();
+       else {
+           KMessageBox::sorry( d->mainWidget,
+                               i18n("You can only select local files."),
+                               i18n("Remote Files Not Accepted") );
+       }
     }
     return QString::null;
 }
@@ -1564,6 +1597,35 @@ QString KFileDialog::getSaveFileName(const QString& dir, const QString& filter,
     return filename;
 }
 
+QString KFileDialog::getSaveFileNameWId(const QString& dir, const QString& filter,
+                                     WId parent_id,
+                                     const QString& caption)
+{
+    bool specialDir = dir.at(0) == ':';
+    QWidget* parent = QWidget::find( parent_id );
+    KFileDialog dlg( specialDir ? dir : QString::null, filter, parent, "filedialog", true);
+#ifdef Q_WS_X11
+    if( parent == NULL && parent_id != 0 )
+        XSetTransientForHint(qt_xdisplay(), dlg.winId(), parent_id);
+#else
+    // TODO
+#endif
+
+    if ( !specialDir )
+        dlg.setSelection( dir ); // may also be a filename
+
+    dlg.setOperationMode( KFileDialog::Saving);
+    dlg.setCaption(caption.isNull() ? i18n("Save As") : caption);
+
+    dlg.exec();
+
+    QString filename = dlg.selectedFile();
+    if (!filename.isEmpty())
+        KRecentDocument::add(filename);
+
+    return filename;
+}
+
 KURL KFileDialog::getSaveURL(const QString& dir, const QString& filter,
                              QWidget *parent, const QString& caption)
 {
@@ -1653,6 +1715,9 @@ void KFileDialog::readConfig( KConfig *kc, const QString& group )
     // show or don't show the speedbar
     toggleSpeedbar( kc->readBoolEntry(ShowSpeedbar, true) );
 
+    // show or don't show the bookmarks
+    toggleBookmarks( kc->readBoolEntry(ShowBookmarks, false) );
+
     // does the user want Automatically Select Extension?
     d->autoSelectExtChecked = kc->readBoolEntry (AutoSelectExtChecked, DefaultAutoSelectExtChecked);
     updateAutoSelectExtension ();
@@ -1681,6 +1746,7 @@ void KFileDialog::writeConfig( KConfig *kc, const QString& group )
     kc->writeEntry( PathComboCompletionMode, static_cast<int>(d->pathCombo->completionMode()) );
     kc->writeEntry( LocationComboCompletionMode, static_cast<int>(locationEdit->completionMode()) );
     kc->writeEntry( ShowSpeedbar, d->urlBar && !d->urlBar->isHidden() );
+    kc->writeEntry( ShowBookmarks, d->bookmarkHandler != 0 );
     kc->writeEntry( AutoSelectExtChecked, d->autoSelectExtChecked );
 
     ops->writeConfig( kc, group );
@@ -1732,6 +1798,11 @@ void KFileDialog::slotCancel()
 {
     ops->close();
     KDialogBase::slotCancel();
+
+    KConfig *config = KGlobal::config();
+    config->setForceGlobal( true );
+    writeConfig( config, ConfigGroup );
+    config->setForceGlobal( false );
 }
 
 void KFileDialog::setKeepLocation( bool keep )
@@ -1751,10 +1822,12 @@ void KFileDialog::setOperationMode( OperationMode mode )
     filterWidget->setEditable( !d->hasDefaultFilter || mode != Saving );
     if ( mode == Opening )
        d->okButton->setGuiItem( KGuiItem( i18n("&Open"), "fileopen") );
-    else if ( mode == Saving )
+    else if ( mode == Saving ) {
        d->okButton->setGuiItem( KStdGuiItem::save() );
+       setNonExtSelection();
+    }
     else
-       d->okButton->setGuiItem( KStdGuiItem::KStdGuiItem::ok() );
+       d->okButton->setGuiItem( KStdGuiItem::ok() );
     updateLocationWhatsThis ();
     updateAutoSelectExtension ();
 }
@@ -2006,8 +2079,31 @@ void KFileDialog::updateLocationEditExtension (const QString &lastExtension)
             fileName.truncate (dot);
 
         // add extension
-        locationEdit->setCurrentText (urlStr.left (fileNameOffset) + fileName + d->extension);
-        locationEdit->lineEdit()->setEdited (true);
+        const QString newText = urlStr.left (fileNameOffset) + fileName + d->extension;
+        if ( newText != locationEdit->currentText() )
+        {
+            locationEdit->setCurrentText (urlStr.left (fileNameOffset) + fileName + d->extension);
+            locationEdit->lineEdit()->setEdited (true);
+        }
+    }
+}
+
+// Updates the filter if the extension of the filename specified in locationEdit is changed
+// (this prevents you from accidently saving "file.kwd" as RTF, for example)
+void KFileDialog::updateFilter ()
+{
+    if ((operationMode() == Saving) && (mode() & KFile::File) ) {
+        const QString urlStr = locationEdit->currentText ();
+        if (urlStr.isEmpty ())
+            return;
+
+        KMimeType::Ptr mime = KMimeType::findByPath(urlStr, 0, true);
+        if (mime && mime->name() != KMimeType::defaultMimeType()) {
+            if (filterWidget->currentFilter() != mime->name() &&
+                filterWidget->filters.findIndex(mime->name()) != -1) {
+                filterWidget->setCurrentFilter(mime->name());
+            }
+        }
     }
 }
 
@@ -2141,6 +2237,41 @@ void KFileDialog::toggleSpeedbar( bool show )
     static_cast<KToggleAction *>(actionCollection()->action("toggleSpeedbar"))->setChecked( show );
 }
 
+void KFileDialog::toggleBookmarks(bool show)
+{
+    if (show)
+    {
+        if (d->bookmarkHandler)
+        {
+            return;
+        }
+
+        d->bookmarkHandler = new KFileBookmarkHandler( this );
+        connect( d->bookmarkHandler, SIGNAL( openURL( const QString& )),
+                    SLOT( enterURL( const QString& )));
+
+        toolbar->insertButton(QString::fromLatin1("bookmark"),
+                              (int)HOTLIST_BUTTON, true,
+                              i18n("Bookmarks"), 5);
+        toolbar->getButton(HOTLIST_BUTTON)->setPopup(d->bookmarkHandler->menu(),
+                                                     true);
+        QWhatsThis::add(toolbar->getButton(HOTLIST_BUTTON),
+                        i18n("<qt>This button allows you to bookmark specific locations. "
+                                "Click on this button to open the bookmark menu where you may add, "
+                                "edit or select a bookmark.<p>"
+                                "These bookmarks are specific to the file dialog, but otherwise operate "
+                                "like bookmarks elsewhere in KDE.</qt>"));
+    }
+    else if (d->bookmarkHandler)
+    {
+        delete d->bookmarkHandler;
+        d->bookmarkHandler = 0;
+        toolbar->removeItem(HOTLIST_BUTTON);
+    }
+
+    static_cast<KToggleAction *>(actionCollection()->action("toggleBookmarks"))->setChecked( show );
+}
+
 int KFileDialog::pathComboIndex()
 {
     return d->m_pathComboIndex;
@@ -2209,6 +2340,22 @@ void KFileDialog::setStartDir( const KURL& directory )
         *lastDirectory = directory;
 }
 
+void KFileDialog::setNonExtSelection()
+{
+    // Enhanced rename: Don't highlight the file extension.
+    QString pattern, filename = locationEdit->currentText().stripWhiteSpace();
+    KServiceTypeFactory::self()->findFromPattern( filename, &pattern );
+
+    if ( !pattern.isEmpty() && pattern.at( 0 ) == '*' && pattern.find( '*' , 1 ) == -1 )
+       locationEdit->lineEdit()->setSelection( 0, filename.length() - pattern.stripWhiteSpace().length()+1 );
+    else
+    {
+       int lastDot = filename.findRev( '.' );
+       if ( lastDot > 0 )
+          locationEdit->lineEdit()->setSelection( 0, lastDot );
+    }
+}
+
 void KFileDialog::virtual_hook( int id, void* data )
 { KDialogBase::virtual_hook( id, data ); }
 
