@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 //#include <stdio.h>
@@ -45,7 +45,7 @@
 class KTar::KTarPrivate
 {
 public:
-    KTarPrivate() : tarEnd( 0 ) {}
+    KTarPrivate() : tarEnd( 0 ), tmpFile( 0 ) {}
     QStringList dirList;
     int tarEnd;
     KTempFile* tmpFile;
@@ -57,12 +57,10 @@ public:
 };
 
 KTar::KTar( const QString& filename, const QString & _mimetype )
-    : KArchive( 0L )
+    : KArchive( 0 )
 {
     m_filename = filename;
     d = new KTarPrivate;
-    d->tmpFile = 0L;
-    d->mimetype = _mimetype;
     QString mimetype( _mimetype );
     bool forced = true;
     if ( mimetype.isEmpty() ) // Find out mimetype manually
@@ -107,8 +105,8 @@ KTar::KTar( const QString& filename, const QString & _mimetype )
             file.close();
         }
         forced = false;
-        d->mimetype = mimetype;
-    } // END mimetype.isEmpty()
+    }
+    d->mimetype = mimetype;
 
     prepareDevice( filename, mimetype, forced );
 }
@@ -144,6 +142,7 @@ void KTar::prepareDevice( const QString & filename,
 KTar::KTar( QIODevice * dev )
     : KArchive( dev )
 {
+    Q_ASSERT( dev );
     d = new KTarPrivate;
 }
 
@@ -259,9 +258,11 @@ Q_LONG KTar::readHeader(char *buffer,QString &name,QString &symlink) {
 
   // if not result of longlink, read names directly from the header
   if (name.isEmpty())
-    name = QFile::decodeName(buffer);
+    // there are names that are exactly 100 bytes long
+    // and neither longlink nor \0 terminated (bug:101472)
+    name = QFile::decodeName(QCString(buffer, 101));
   if (symlink.isEmpty())
-    symlink = QFile::decodeName(buffer + 0x9d);
+    symlink = QFile::decodeName(QCString(buffer + 0x9d, 101));
 
   return 0x200;
 }
@@ -340,6 +341,9 @@ bool KTar::openArchive( int mode )
     d->dirList.clear();
     QIODevice* dev = device();
 
+    if ( !dev )
+        return false;
+
     // read dir infos
     char buffer[ 0x200 ];
     bool ende = false;
@@ -390,7 +394,7 @@ bool KTar::openArchive( int mode )
             // '0' for files, '1' hard link, '2' symlink, '5' for directory
             // (and 'L' for longlink filenames, 'K' for longlink symlink targets)
             // and 'D' for GNU tar extension DUMPDIR
-            if ( typeflag == '1' )
+            if ( typeflag == '5' )
                 isdir = true;
 
             bool isDumpDir = false;
@@ -408,7 +412,7 @@ bool KTar::openArchive( int mode )
             KArchiveEntry* e;
             if ( isdir )
             {
-                //kdDebug(7041) << "KArchive::open directory " << nm << endl;
+                //kdDebug(7041) << "KTar::openArchive directory " << nm << endl;
                 e = new KArchiveDirectory( this, nm, access, time, user, group, symlink );
             }
             else
@@ -423,10 +427,11 @@ bool KTar::openArchive( int mode )
                 // for isDumpDir we will skip the additional info about that dirs contents
                 if ( isDumpDir )
                 {
-		    e = new KArchiveDirectory( this, nm, access, time, user, group, symlink );
+                    //kdDebug(7041) << "KTar::openArchive " << nm << " isDumpDir" << endl;
+                    e = new KArchiveDirectory( this, nm, access, time, user, group, symlink );
                 }
-		else
-		{
+                else
+                {
 
                     // Let's hack around hard links. Our classes don't support that, so make them symlinks
                     if ( typeflag == '1' )
@@ -435,17 +440,17 @@ bool KTar::openArchive( int mode )
                         kdDebug(7041) << "HARD LINK, setting size to " << size << endl;
                     }
 
-                    // kdDebug(7041) << "KArchive::open file " << nm << " size=" << size << endl;
+                    //kdDebug(7041) << "KTar::openArchive file " << nm << " size=" << size << endl;
                     e = new KArchiveFile( this, nm, access, time, user, group, symlink,
                                           dev->at(), size );
-		}
+                }
 
                 // Skip contents + align bytes
                 int rest = size % 0x200;
                 int skip = size + (rest ? 0x200 - rest : 0);
-                //kdDebug(7041) << "KArchive::open, at()=" << dev->at() << " rest=" << rest << " skipping " << skip << endl;
+                //kdDebug(7041) << "KTar::openArchive, at()=" << dev->at() << " rest=" << rest << " skipping " << skip << endl;
                 if (! dev->at( dev->at() + skip ) )
-                    kdWarning(7041) << "KArchive::open skipping " << skip << " failed" << endl;
+                    kdWarning(7041) << "KTar::openArchive skipping " << skip << " failed" << endl;
             }
 
             if ( pos == -1 )
