@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -285,6 +285,22 @@ KConfig *cfg;
    if (!libname.isNull())
          _cryptoLib = ll->globalLibrary(libname.latin1());
    }
+#elif defined(__CYGWIN__)
+   libpaths << "/usr/bin/"             
+   		<< "/usr/local/bin"             
+   		<< "/usr/local/openssl/bin"     
+   		<< "/opt/openssl/bin"           
+   		<< "/opt/kde3/bin"              
+   		<< "";                          
+                                       
+   libnamess << "cygssl-0.9.7.dll"     
+		 << "cygssl.dll"                    
+		 << "libssl.dll"                    
+		 << "";                         
+                                       
+   libnamesc << "cygcrypto.dll"        
+		 << "libcrypto.dll"                 
+		 << "";                         
 #else
    libpaths
             #ifdef _AIX
@@ -298,7 +314,7 @@ KConfig *cfg;
 	    << "/opt/openssl/lib" KDELIBSUFF "/"
 	    << "/lib" KDELIBSUFF "/"
             << "";
-
+    
 // FIXME: #define here for the various OS types to optimize
    libnamess
 	     #ifdef hpux
@@ -333,6 +349,7 @@ KConfig *cfg;
 	     << "libcrypto.so.0"
              #endif
 	     ;
+#endif
 
    for (QStringList::Iterator it = libpaths.begin();
                               it != libpaths.end();
@@ -352,7 +369,6 @@ KConfig *cfg;
       }
       if (_cryptoLib) break;
    }
-#endif
 
    if (_cryptoLib) {
 #ifdef KSSL_HAVE_SSL
@@ -560,21 +576,31 @@ KConfig *cfg;
       if (_cryptoLib) {
          if (x) ((int (*)())x)();
          x = _cryptoLib->symbol("OpenSSL_add_all_algorithms");
+         if (!x)
+            x = _cryptoLib->symbol("OPENSSL_add_all_algorithms");
          if (x) {
            ((void (*)())x)();
          } else {
            x = _cryptoLib->symbol("OpenSSL_add_all_algorithms_conf");
+           if (!x)
+              x = _cryptoLib->symbol("OPENSSL_add_all_algorithms_conf");
            if (x) {
              ((void (*)())x)();
            } else {
              x = _cryptoLib->symbol("OpenSSL_add_all_algorithms_noconf");
+             if (!x)
+                x = _cryptoLib->symbol("OPENSSL_add_all_algorithms_noconf");
              if (x)
                ((void (*)())x)();
            }
          }
          x = _cryptoLib->symbol("OpenSSL_add_all_ciphers");
+         if (!x)
+            x = _cryptoLib->symbol("OPENSSL_add_all_ciphers");
          if (x) ((void (*)())x)();
          x = _cryptoLib->symbol("OpenSSL_add_all_digests");
+         if (!x)
+            x = _cryptoLib->symbol("OPENSSL_add_all_digests");
          if (x) ((void (*)())x)();
       }
    }
