@@ -13,8 +13,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <qapplication.h>
@@ -80,13 +80,13 @@ class KCModuleProxy::KCModuleProxyPrivate
 
 		~KCModuleProxyPrivate()
 		{
+			delete rootInfo; // Delete before embedWidget!
+			delete embedWidget; // Delete before embedFrame!
 			delete embedFrame;
 			delete dcopClient;
 			delete dcopObject;
-			delete embedWidget;
 			delete rootCommunicator;
 			delete rootProcess;
-			delete rootInfo;
 			delete kcm;
 		}
 
@@ -386,11 +386,10 @@ void KCModuleProxy::runAsRoot()
 	}
 
 	/* Clean up in case of failure */
-	delete d->embedFrame;
-	d->embedFrame = 0;
 	delete d->embedWidget;
 	d->embedWidget = 0;
-	delete lblBusy;
+	delete d->embedFrame;
+	d->embedFrame = 0;
 
 	QApplication::restoreOverrideCursor();
 }
